import base64, zlib



class AriesNode(list):
    def __init__(self, name=''):
        super(AriesNode, self).__init__()

        self.name = name

    def __getitem__(self, name):
        if isinstance(name, basestring):
            for child in self:
                if isinstance(child, AriesNode) and child.name == name:
                    return child

            else:
                raise KeyError

        else:
            return super(AriesNode, self).__getitem__(name)

    def __str__(self, indent=0):
        i = '\t' * indent
        if not len(self):
            return '(%s)' % self.name

        elif len(self) == 1 and not isinstance(self[0], AriesNode):
            return "(%s '%s')" % (self.name, self[0])

        else:
            children = '\n'
            for child in self:
                children += i + '\t'
                try:
                    children += child.__str__(indent + 1)
                except TypeError:
                    children += str(child)
                children += '\n'
            return '(%s %s%s)' % (self.name, children, i)

    __repr__ = __str__


    def get_value(self, key):
        return self[key][0]

    def get_data(self, name, format):
        child = self.get(key)

        if child.get('format') == format:
            if format == 'zlib':
                data = child[1]
                data = base64.b64decode(data)

                zd = zlib.decompressobj()
                data = zd.decompress(data)

                return data


class AriesFile(file):
    def __init__(self, path, mode='r'):
        super(AriesFile, self).__init__(path, mode)

        self.source = super(AriesFile, self).read()
        self.length = len(self.source)

        self.__pos = 0

    def write(self, node):
        super(AriesFile, self).write(str(node))

    def read(self):
        root = AriesNode("")
        L = [root]

        while self.__pos < self.length:
            c = self.__get()

            if c.isspace():
                continue

            elif c == '(':
                node = AriesNode(self.__grab_identifier())

                L[-1].append(node)
                L.append(node)

            elif c == ')':
                assert len(L) > 1, \
                    'Malformed markup document: mismatched end parenthesis.'
                L.pop()

            else:
                L[-1].append(self.__grab_string().rstrip())

        return root


    def __get(self):
        self.__pos += 1

        return self.source[self.__pos - 1]

    def __grab(self, symbols):
        o = []
        for s in symbols:
            n = self.source[self.__pos:].find(s)
            if n == -1:
                n = self.length
            o.append(n)

        return min(*o)

    def __grab_identifier(self):
        old = self.__pos
        self.__pos += self.__grab(' \n\t()')

        return self.source[old:self.__pos]

    def __grab_string(self):
        old = self.__pos
        self.__pos += self.__grab('()')

        return self.source[old:self.__pos - 1]
