import ika
import igl



class BGM(object):
    def __init__(self):
        self.current = None
        self.previous = None

        self.cur_fade = 0

    def start_fade(self, ticks=None):
        if ticks is None:
            ticks = 50

        self.cur_fade = ticks
        self.max_fade = ticks
        self.fade_inc = 1.0 / ticks

    def save(self):
        self.previous = self.current

    def play(self, name=None, loop=True):
        if name is None:
            if self.current:
                self.previous = self.current
                self.current = None
                self.start_fade()

            return

        if isinstance(name, basestring):
            song = ika.Music("music/" + name)
        else:
            song = name

        song.loop = loop

        if self.current:
            if self.current[0] == name:
                return

        self.current = (name, song)
        self.start_fade()
        song.Play()

    def pause(self):
        if self.current:
            self.current.Pause()

    def revert(self):
        c = self.current
        self.current = self.previous
        self.previous = c
        self.start_fade()

    def update(self):
        if self.cur_fade and (self.current or self.previous):
            if self.current:
                t = self.max_fade - self.cur_fade
                self.current[1].volume = min(t * self.fade_inc, 1)

            if self.previous:
                song = self.previous[1]
                song.volume = max(self.cur_fade * self.fade_inc, 0)
                if song.volume == 0:
                    song.Pause()

            self.cur_fade -= 1

bgm = BGM()
igl.controller.attach(bgm.update)
