import widgets
import models

class Field(object):
    empty_strings_allowed = True
    creation_counter = 0
    primary = False
    private = False #Indicates whether it's editable by the user or not.
    inherit = True
    widget = None

    def __init__(self, default=None, null=False):
        self.name = None
        self.default = default
        self.null = null

        self.creation_counter = Field.creation_counter
        Field.creation_counter += 1

    def contribute_to_class(self, cls, name):
        self.name = name
        self.db_name = name
        cls.add_field(name, self)


    def get_db_name(self):
        return self.name

    def get_default(self):
        default = self.default

        if callable(default):
            return default()

        else:
            return default

    def to_python(self, value):
        return value

    def to_db(self, value):
        return value

    def from_db(self, value):
        return value



class AutoField(Field):
    db_type = 'INTEGER'
    primary = True
    inherit = False

class CategoryField(Field):
    private = True
    db_type = 'INTEGER'
    inherit = False

class IntegerField(Field):
    db_type = 'INTEGER'
    empty_strings_allowed = False

    def __init__(self, default=0, null=False, max=0, min=0, increment=1):
        self.widget = widgets.NumberBox(max, min, increment, 0)

        Field.__init__(self, default, null)

    def to_python(self, value):

        try:
            return int(value)

        except:
            raise Exception, u'This value must be an integer'

class FloatField(IntegerField):
    db_type = 'REAL'
    empty_strings_allowed = False

    def __init__(self, default=0.0, null=False, max=0, min=0, increment=.1, digits=1):
        self.widget = widgets.NumberBox(max, min, increment, digits)

        Field.__init__(self, default, null)

    def to_python(self, value):
        print value, type(value)
        try:
            return float(value)

        except:
            raise Exception, u'This value must be a float'

class EnumField(IntegerField):

    def __init__(self, choices, default=0, null=False):

        self.widget = widgets.ChoiceBox(choices)

        Field.__init__(self, default, null)

class BooleanField(Field):

    db_type = 'INTEGER'
    empty_strings_allowed = False

    def __init__(self, default=0, null=False):

        self.widget = widgets.CheckBox()

        Field.__init__(self, default, null)

    def to_python(self, value):

        try:
            return int(value)

        except:
            print value
            raise Exception, u'This value must be an integer'

class StringField(Field):
    db_type = 'TEXT'

    def __init__(self, default=u'', null=False, maxlength=None, width=200, height=-1, multiline=False):

        self.widget = widgets.TextBox(maxlength, width, height, multiline)

        Field.__init__(self, default, null)

    def to_python(self, value):
        try:
            return unicode(value)

        except:
            raise Exception, u'This value must be a string'



class ForeignField(Field):
    db_type = 'TEXT'
    #ChildrenAllowed, ParentModel are currently UNSUPPORTED
    #Since they require trawling through the entire model list which isn't
    #possible within the DB library itself (yet).
    def __init__(self, model_types, childrenallowed=True, newmodel=False, newonly=False, parentmodel=False, editable=True):
        self.model = None
        self.model_types = model_types

        self.widget = widgets.ForeignModel(model_types, childrenallowed, newmodel, newonly, parentmodel,editable)

        Field.__init__(self, default='', null=True)

    def from_db(self, value):
        if value is not None and value != '' and value != 'Null':
            model, id = value.split(':')
            id = int(id)
            for x in self.model_types[0].model_cache[model].objects:
                if x.id == id:
                    return x
            #return self.model_types[0].model_cache[v[0]].objects.get(v[1])

        return None

    def to_db(self, value):
        #print 'VALUE: '+value
        if (value is not None and value != ""):
            return "%s:%s" % (value.__class__.__name__, str(value.id))
        else:
            return 'Null'

class MultiForeignField(Field):
    db_type = 'TEXT'
    #ChildrenAllowed, ParentModel are currently UNSUPPORTED
    #Since they require trawling through the entire model list which isn't
    #possible within the DB library itself (yet).
    def __init__(self, modeltypes, width=180, height=180, childrenallowed=True, newmodel=False, newonly=False, parentmodel=False, editable=True, max=0):
        self.models = []
        self.model_types = modeltypes

        self.widget = widgets.MultiForeignModel(modeltypes, width, height, childrenallowed, newmodel, newonly, parentmodel,editable, max)

        Field.__init__(self, default='', null=True)

    def from_db(self, value):

        #print 'VALUE: '+str(value)

        if value is not None:
            list = value.split(',')
            final_list = []
            for pair in list:
                #print "PAIR:"+pair
                if pair != '':
                    model, id = pair.split(':')
                    #print "MODEL:"+model
                    
                    #for x in self.model_types[0].model_cache[model].objects:
                        #print "COMPAR "+str(x.id)+"-"+str(id)
                    #    if str(x.id) == str(id):
                            #print "MATCH"
                    #        final_list.append(x)

		    m =  self.model_types[0].model_cache[model].objects.get(id)
		    if m != None:
		         final_list.append( m )

                
        #print 'FINAL LIST:'+str(final_list)
                
        return final_list

    def to_db(self, value):

        final_string = ''
        
        if value is not []:

            for x in value:
                final_string = final_string+"%s:%s," % (x.__class__.__name__, str(x.id))

            #print "FINAL STRING: "+final_string[:-1]

            return final_string[:-1]
        
        else:
            
            return 'Null'
