import ika



class ProcessController(object):
    def __init__(self):
        self.__attachments = []

        self.processes = []
        self.running = False

        self.__to_set = None
        self.__to_push = []
        self.__to_pop = False

        self.last_update = None


    def attach(self, method):
        if method not in self.__attachments:
            self.__attachments.append(method)

    def detach(self, method):
        if method in self.__attachments:
            self.__attachments.remove(method)

    def is_attached(self, method):
        return method in self.__attachments



    def set(self, new_process):
        if self.processes:
            self.__to_set = new_process

        else:
            self.processes = [new_process]

    def push(self, process):
        self.__to_push.append(process)

    def pop(self):
        self.__to_pop = True



    def update(self):
        now = ika.GetTime()
        if self.last_update is None:
            updates = 1

        else:
            updates = (now - self.last_update)


        if updates:
            ika.Input.Update()

            self.last_update = now

            for i in xrange(updates):
                # Update all our attachments.
                [a() for a in self.__attachments]

                # Update our current process.
                [p.update() for p in self.processes]

    def render(self):
        [p.render() for p in self.processes]

        if True:
            ika.Video.ShowPage()

    def tick(self):
        self.update()
        self.render()

        self.process_changes()

    def process_changes(self):
        if self.__to_set:
            [old_process.end() for old_process in self.processes]

            self.processes = [self.__to_set]
            new_process.start()

            self.__to_set = None

        elif self.__to_push:
            for p in self.__to_push:
                self.processes.insert(0, p)
                p.start()

            self.__to_push = []

        elif self.__to_pop:
            process = self.processes.pop()
            process.end()

            self.__to_pop = False


    def run(self):
        if not self.running:
            self.running = True

            self.process_changes()

            while self.processes and self.running:
                self.tick()

            ika.Exit()



class Process(object):
    def start(self):
        pass

    def end(self):
        pass

    def update(self):
        pass

    def render(self):
        pass
