import ika
from igl import utils

__ALL__ = ['WindowSkin', 'WINDOW_TILE', 'WINDOW_STRETCH']

WINDOW_TILE, WINDOW_STRETCH = range(2)



class WindowSkin(object):
    def __init__(self, path, background_style=WINDOW_STRETCH, foreground_style=WINDOW_TILE, padding=0):
        self.path = path

        self.background_style = background_style
        self.foreground_style = foreground_style
        self.padding = padding

        #
        #   Load images
        #
        # Load background
        try:
            image = ika.Image(path + "background.png")
        except:
            image = None
        self.background_image = image

        # Load border
        try:
            image = utils.images.get_grid(path + "border.png", 3, 3)

            # Save extra details on the border images.
            self.border_size = (image[0].width, image[0].height)
        except:
            image = None
        self.border_image = image

        # Load foreground
        try:
            image = ika.Image(path + "foreground.png")
        except:
            image = None
        self.foreground_image = image

    #
    #   Property methods for background.
    #
    def __get_background_style(self):
        return self.__background_style

    def __set_background_style(self, style):
        self.__background_style = style
        self.background_method = drawing_methods[style]

    #
    #   Property methods for foreground.
    #
    def __get_foreground_style(self):
        return self.__foreground_style

    def __set_foreground_style(self, style):
        self.__foreground_style = style
        self.foreground_method = drawing_methods[style]

    background_style = property(__get_background_style, __set_background_style)
    foreground_style = property(__get_foreground_style, __set_foreground_style)


    def draw(self, x, y, w, h):
        p = self.padding

        # Draw background
        if self.background_image:
            self.background_method(self.background_image, x, y, w, h)

        # Draw foreground
        if self.foreground_image:
            self.foreground_method(self.foreground_image, x, y, w, h)

        # Draw border
        if self.border_image:
            images = self.border_image
            bw, bh = self.border_size

            # Draw corners.
            x2 = x - p
            y2 = y - p
            x3 = x + p + w - bw
            y3 = y + p + h - bh
            images[0].Blit(x2, y2)
            images[2].Blit(x3, y2)
            images[6].Blit(x2, y3)
            images[8].Blit(x3, y3)

            # Draw edges.
            bw2 = w - bw * 2 + p * 2
            bh2 = h - bh * 2 + p * 2
            images[1].TileBlit(x2 + bw, y2, bw2, bh)
            images[7].TileBlit(x2 + bw, y3, bw2, bh)
            images[3].TileBlit(x2, y2 + bh, bw, bh2)
            images[5].TileBlit(x3, y2 + bh, bw, bh2)


    @staticmethod
    def draw_tiled(image, x, y, w, h):
        #image.ScaleBlit(x, y, w, h)
        ika.Video.TileBlit(image, x, y, w, h)

    @staticmethod
    def draw_stretched(image, x, y, w, h):
        image.ScaleBlit(x, y, w, h)


drawing_methods = {
    WINDOW_TILE: WindowSkin.draw_tiled,
    WINDOW_STRETCH: WindowSkin.draw_stretched
}
