import ika

def Brensenham_line(p1,p2):
	"""Brensenham line algorithm"""
	x = p1[0]
	y = p1[1]
	x2 = p2[0]
	y2 = p2[1]
	
	steep = 0
	coords = []
	dx = abs(x2 - x)
	if (x2 - x) > 0: sx = 1
	else: sx = -1
	dy = abs(y2 - y)
	if (y2 - y) > 0: sy = 1
	else: sy = -1
	if dy > dx:
		steep = 1
		x,y = y,x
		dx,dy = dy,dx
		sx,sy = sy,sx
	d = (2 * dy) - dx
	for i in range(0,dx):
		if steep: coords.append((y,x))
		else: coords.append((x,y))
		while d >= 0:
			y = y + sy
			d = d - (2 * dx)
		x = x + sx
		d = d + (2 * dy)
	coords.append((x2,y2))
	coords.pop()
	return coords

class Point:
	
	def __init__(self, x, y):
		
		self.x = x
		self.y = y
		
	def __str__(self):
		
		return "(%s,%s)" % (str(self.x), str(self.y))
		
class Cliff:
	
	def __init__(self,x,y,w,h,points,layers,height,caves,ladders):
		
		self.x = x
		self.y = y
		self.w = w
		self.h = h
		
		self.PointN = points
		self.Layers = layers
		self.Height = height
		self.Caves = caves
		self.Ladders = ladders
		
		self.Points = []
		
	def Generate(self):
		
		for x in range(self.Layers):
			self.GenerateLayer()
			for x in range(10):
				self.SmoothLayer()
			self.ApplyEdges()
			self.ApplyShadows()
	
	def GenerateLayer(self):

		Low_X = []
		Low_Y = []
		High_X = []
		High_Y = []
		
		if len(self.Points) == 0:
		
			for x in range(0,self.PointN):
				p = Point(ika.Random(self.x,self.x+self.w), ika.Random(self.y,self.y+self.h) )
				self.Points.append(  p )
			
		self.Points.sort(SortX)
		Low_X = self.Points[0:2]
		Low_X.sort(SortY)
		
		self.Points.sort(SortX, reverse=True)
		High_X = self.Points[0:2]
		High_X.sort(SortY, reverse=True)
		
		self.Points.sort(SortY)
		Low_Y = self.Points[0:2]
		Low_Y.sort(SortX, reverse=True)
		
		self.Points.sort(SortY, reverse=True)
		High_Y = self.Points[0:2]
		High_Y.sort(SortX)
			
		Final = Low_X+High_Y+High_X+Low_Y+Low_X[:1]
		
		previous = None
		for x in Final:
			#print str(x)+"\n"
			
			if previous != None:
				DrawLine(previous, x)
			
			previous = x
			
		for x in Final:
			del x
		
	def SmoothLayer(self):	
		
		GetTile = self.GetTile
		SetTile = self.SetTile
		
		for x in range(self.x,self.x+self.w):
			for y in range(self.y,self.y+self.h):
				if GetTile(x,y,0) == 1:
					
					while self.AdjacentCount(x,y) < 2:
						Option = [9,9,9,9]
						
						if self.GetTile(x+1,y,0) == 0 and self.AdjacentCount(x+1,y) >= 2:
							Option[0] = self.AdjacentCount(x+1,y)
						elif self.GetTile(x-1,y,0) == 0 and self.AdjacentCount(x-1,y) >= 2:
							Option[1] = self.AdjacentCount(x-1,y)
						elif self.GetTile(x,y+1,0) == 0 and self.AdjacentCount(x,y+1) >= 2:
							Option[2] = self.AdjacentCount(x,y+1)
						elif self.GetTile(x,y-1,0) == 0 and self.AdjacentCount(x,y-1) >= 2:
							Option[3] = self.AdjacentCount(x,y-1)

							
						O = min(Option)
						
						if Option[0] == O:
							self.SetTile(x+1,y,0,1)
						elif Option[1] == O:
							self.SetTile(x-1,y,0,1)
						elif Option[2] == O:
							self.SetTile(x,y+1,0,1)
						elif Option[3] == O:
							self.SetTile(x,y-1,0,1)
							
							
		#for x in range(self.x,self.x+self.w):
		#	for y in range(self.y,self.y+self.h):
		#		if GetTile(x,y,0) == 1:
		#			if self.AdjacentCount(x,y) != 2:
		#				self.SetTile(x,y,0,0)
						

	def AdjacentCount(self,x,y):
		
		c = 0
		
		if self.GetTile(x+1,y,0) != 0:
			c+=1
		if self.GetTile(x-1,y,0) != 0:
			c+=1
		if self.GetTile(x,y+1,0) != 0:
			c+=1
		if self.GetTile(x,y-1,0) != 0:
			c+=1			
			
		return c
		
	def ApplyEdges(self):
		
		GetTile = self.GetTile
		SetTile = self.SetTile
		
		for x in range(self.x,self.x+self.w):
			for y in range(self.y,self.y+self.h):
				if GetTile(x,y,0) == 1:
					
					#Forward-facing cliff check.
					#If there are cliffs to the left and right of this cliff and there's nothing below it.
					if GetTile(x-1,y,0) == 1 and GetTile(x+1,y,0) == 1 and not self.TilesBelow(x,y):
						SetTile(x,y,1,42)
						
					#Forward-facing right edge cliff
					#If there's no cliff to the right but one to the left or bottom left. If it's bottom left, place a side right cliff next to us. If one is at all beneath, then not.
					if GetTile(x+1,y,0) == 0 and not self.TilesBelow(x,y):
						if GetTile(x-1,y,0) == 1 or GetTile(x-1,y+1,0) == 1:
							SetTile(x,y,1,45)
						
					#Right-side cliff
					#If there are cliffs above and below this cliff.
					#if GetTile(x,y+1,0) == 1 and GetTile(x,y-1,0) == 1:
					#	SetTile(x,y,1,49)
				
		
	def GetTile(self,x,y,l):
		return ika.Map.GetTile(x,y,l)
		
	def SetTile(self,x,y,l,t):
		ika.Map.SetTile(x,y,l,t)
		
	def TilesBelow(self,x,y):
		y+=1
		
		while y < self.y+self.h:
			if self.GetTile(x,y,0) != 0:
				return 1
			
			y+=1
			
		return 0
		
		
		
	def ApplyShadows(self):
		pass
		
		
def SortX(a,b):
	
	if a.x > b.x:
		return 1
	elif a.x < b.x:
		return -1
	elif a.x == b.x:
		return 0

def SortY(a,b):
	
	if a.y > b.y:
		return 1
	elif a.y < b.y:
		return -1
	elif a.y == b.y:
		return 0	
		
def GenerateMap():
	
	#(x,y,w,h,points,layers,height,caves,ladders)
	
	c1 = Cliff(0,0,5,5,
			15, #Points 
			1, #Layers
			1, #Height
			0, #Caves
			0) #Ladders
			
	c2 = Cliff(6,0,9,9,
			15, #Points 
			1, #Layers
			1, #Height
			0, #Caves
			0) #Ladders

	c1.Generate()
	c2.Generate()

			
	
def DrawLine(p1, p2):
	
	p1 = [p1.x, p1.y]
	p2 = [p2.x, p2.y]
	
	for x,y in Brensenham_line(p1, p2):
		ika.Map.SetTile(x,y,0,1)	
	
def CheckLineObs(p1,p2):
	
	p1 = [p1.x, p1.y]
	p2 = [p2.x, p2.y]	
	
	obsC = 0
	
	for x,y in Brensenham_line(p1, p2):
		if ika.Map.GetTile(x,y,0) == 1:
			obsC+=1
			
	if obsC > 1:
		return True
	else:
		return False
			
	