import ika

def Brensenham_line(p1,p2):
	"""Brensenham line algorithm"""
	x = p1[0]
	y = p1[1]
	x2 = p2[0]
	y2 = p2[1]
	
	steep = 0
	coords = []
	dx = abs(x2 - x)
	if (x2 - x) > 0: sx = 1
	else: sx = -1
	dy = abs(y2 - y)
	if (y2 - y) > 0: sy = 1
	else: sy = -1
	if dy > dx:
		steep = 1
		x,y = y,x
		dx,dy = dy,dx
		sx,sy = sy,sx
	d = (2 * dy) - dx
	for i in range(0,dx):
		if steep: coords.append((y,x))
		else: coords.append((x,y))
		while d >= 0:
			y = y + sy
			d = d - (2 * dx)
		x = x + sx
		d = d + (2 * dy)
	coords.append((x2,y2))
	coords.pop()
	return coords

class Point:
	
	def __init__(self, x, y):
		
		self.x = x
		self.y = y
		
	def __str__(self):
		
		return "(%s,%s)" % (str(self.x), str(self.y))
		
class Cliff:
	
	def __init__(self,x,y,w,h,attempts,layers,height,caves,ladders):
		
		self.x = x
		self.y = y
		self.w = w
		self.h = h
		
		self.Attempts = attempts
		self.Layers = layers
		self.Height = height
		self.Caves = caves
		self.Ladders = ladders
		
		self.Recursion = 0
		
		self.BadTiles = []
		
		self.Points = []
		
	def Generate(self):
		
		for x in range(self.Layers):
			if not self.GenerateLayer(x):
				self.BadTiles = []
				for x2 in range(self.x+self.w+max(self.Height)):
					for y2 in range(self.y+self.h):
						ika.Map.SetTile(x2,y2,0,0)
						ika.Map.SetTile(x2,y2,1,68)
						ika.Map.SetTile(x2,y2,2,68)
						ika.Map.SetTile(x2,y2,3,68)
				self.Generate()	
				return 0
				
			#for x in range(10):
			#	self.SmoothLayer()
			if not self.ApplyEdges(x):
				self.BadTiles = []
				for x2 in range(self.x+self.w+max(self.Height)):
					for y2 in range(self.y+self.h):
						ika.Map.SetTile(x2,y2,0,0)
						ika.Map.SetTile(x2,y2,1,68)
						ika.Map.SetTile(x2,y2,2,68)
						ika.Map.SetTile(x2,y2,3,68)
				self.Generate()
				return 0

			for x in range(self.x+self.w):
				for y in range(self.y+self.h):	
					if self.GetTile(x,y,0) == 1:
						self.BadTiles.append( (x,y) )
					self.SetTile(x,y,0,0)
			#self.ApplyShadows()
	
	def GenerateLayer(self, layerN):

		if layerN == 0:
			x = ika.Random(self.x,self.x+5)
			y = ika.Random(self.y,self.y+5)
		else:
			
			x = ika.Random(self.x, self.x+(self.w))
			y = ika.Random(self.y, self.y+(self.h))
			c=0
			while (x,y) in self.BadTiles or not self.InsideBadTiles(x,y):
				c+=1
				x = ika.Random(self.x, self.x+(self.w))
				y = ika.Random(self.y, self.y+(self.h))	

				if c>=1000:
					return 0
		#x = self.x+(layerN*2)
		#y = self.y+(layerN*2)
		
		ika.Map.SetTile(x,y,0,1)
		
		print '---------------------------'
		
		ox = x
		oy = y
		
		A = range(10,15)
		B = range(15,20)
		
		#Go to the right
		#print 'Right'
		Active = ika.Random(self.Attempts,self.Attempts*1.5)
		while x < self.x+self.w and Active > 0:
			#print str((x,y))
			Active-=1
			(x,y) = self.PlaceRandom(0,20,[-1],range(0,10),A,B,(x,y))
		
		#Go down
		#print 'Down'
		Active = ika.Random(self.Attempts,self.Attempts*1.5)
		
		while y < self.y+self.h and Active > 0:
			#print str((x,y))
			Active -= 1
			(x,y) = self.PlaceRandom(0,20,A,B,[-1],range(0,10),(x,y))
			
			
		#Go left
		#print 'Left'
		Active = ika.Random(self.Attempts,self.Attempts*1.5)
		while x >= self.x and Active > 0:
			#print str((x,y))
			Active -= 1
			(x,y) = self.PlaceRandom(0,20,range(0,10),[-1],A,B,(x,y))
			
		#Go up
		#print 'Up'
		Active = ika.Random(self.Attempts,self.Attempts*1.5)
		while (x != ox or y != oy) and Active > 0:
			#print str((x,y))
			Active -=1
			(x,y) = self.PlaceRandom(0,20,A,B,range(0,10),[-1],(x,y))
			
		#print str(self.AdjacentCount(ox+1,oy))	
		#print str(self.AdjacentCount(ox-1,oy))	
		#print str(self.AdjacentCount(ox,oy+1))	
		#print str(self.AdjacentCount(ox,oy-1))	
			
		if self.AdjacentCount(ox+1,oy) == 2 and self.GetTile(ox+1,oy,0) == 0:
			ika.Map.SetTile(ox+1,oy,0,1)
		elif self.AdjacentCount(ox-1,oy) == 2 and self.GetTile(ox-1,oy,0) == 0:
			ika.Map.SetTile(ox-1,oy,0,1)
		elif self.AdjacentCount(ox,oy+1) == 2 and self.GetTile(ox,oy+1,0) == 0:
			ika.Map.SetTile(ox,oy+1,0,1)
		elif self.AdjacentCount(ox,oy-1) == 2 and self.GetTile(ox,oy-1,0) == 0:
			ika.Map.SetTile(ox,oy-1,0,1)
			
		redo = 0
		count = 0
		for x in range(self.x,self.x+self.w):
			for y in range(self.y,self.y+self.h):
				if self.GetTile(x,y,0) == 1:
					count+=1
					if self.AdjacentCount(x,y) != 2:
						redo = 1

		if count < 8:
			redo = 1

		if redo:
			for x in range(self.x+self.w):
				for y in range(self.y+self.h):	
					self.SetTile(x,y,0,0)
					
			self.Recursion+=1
			if self.Recursion >= 200:
				return 0
			return self.GenerateLayer(layerN)

			
		self.Recursion = 0
		return 1

					
	def PlaceRandom(self,min,max,left,right,up,down,xy):
		PlaceTile = self.PlaceTile
		
		x,y = xy
		
		r = ika.Random(min,max)
		
		if r in left:
			#print 'l'
			if PlaceTile(x-1,y):
				x -= 1
		if r in right:
			#print 'r'
			if PlaceTile(x+1,y):
				x += 1
		if r in up:
			#print 'u'
			if PlaceTile(x,y-1):
				y -= 1
		if r in down:
			#print 'd'
			if PlaceTile(x,y+1):
				y += 1
				
		return (x,y)
					
	def PlaceTile(self,x,y):
		if x >= self.x and x < self.x+self.w and y >= self.y and y < self.y+self.h:
			if (self.AdjacentCount(x,y) == 1) and (x,y) not in self.BadTiles:
				self.SetTile(x,y,0,1)
				return 1
		return 0
				
	def SmoothLayer(self):	
		
		GetTile = self.GetTile
		SetTile = self.SetTile

		for x in range(self.x,self.x+self.w):
			for y in range(self.y,self.y+self.h):
				if GetTile(x,y,0) == 1:
					
					while self.AdjacentCount(x,y) < 2:
						Option = [9,9,9,9]
						
						if self.GetTile(x+1,y,0) == 0 and self.AdjacentCount(x+1,y) >= 2:
							Option[0] = self.AdjacentCount(x+1,y)
						elif self.GetTile(x-1,y,0) == 0 and self.AdjacentCount(x-1,y) >= 2:
							Option[1] = self.AdjacentCount(x-1,y)
						elif self.GetTile(x,y+1,0) == 0 and self.AdjacentCount(x,y+1) >= 2:
							Option[2] = self.AdjacentCount(x,y+1)
						elif self.GetTile(x,y-1,0) == 0 and self.AdjacentCount(x,y-1) >= 2:
							Option[3] = self.AdjacentCount(x,y-1)

							
						O = min(Option)
						
						if Option[0] == O:
							self.SetTile(x+1,y,0,1)
						elif Option[1] == O:
							self.SetTile(x-1,y,0,1)
						elif Option[2] == O:
							self.SetTile(x,y+1,0,1)
						elif Option[3] == O:
							self.SetTile(x,y-1,0,1)
							
							
		#for x in range(self.x,self.x+self.w):
		#	for y in range(self.y,self.y+self.h):
		#		if GetTile(x,y,0) == 1:
		#			if self.AdjacentCount(x,y) != 2:
		#				self.SetTile(x,y,0,0)
						

	def AdjacentCount(self,x,y):
		
		c = 0
		
		if x+1 >= self.x and x+1 < self.x+self.w and y >= self.y and y < self.y+self.h:
			if self.GetTile(x+1,y,0) != 0:
				c+=1
				
		if x-1 >= self.x and x-1 < self.x+self.w and y >= self.y and y < self.y+self.h:		
			if self.GetTile(x-1,y,0) != 0:
				c+=1
		if x >= self.x and x < self.x+self.w and y+1 >= self.y and y+1 < self.y+self.h:
			if self.GetTile(x,y+1,0) != 0:
				c+=1
				
		if x >= self.x and x < self.x+self.w and y-1 >= self.y and y-1 < self.y+self.h:
			if self.GetTile(x,y-1,0) != 0:
				c+=1			
			
		return c
		
	def ApplyEdges(self, layerN):
		
		GetTile = self.GetTile
		SetTile = self.SetTile
		
		L = 1+layerN
		
		redo = 0
		
		for x in range(self.x,self.x+self.w):
			for y in range(self.y,self.y+self.h):
				if GetTile(x,y,0) == 1:
					print "(%s,%s)" % (x,y)
					#Forward-facing cliff check.
					#If there are cliffs to the left and right of this cliff and the area below it isn't internal
					if GetTile(x-1,y,0) == 1 and GetTile(x+1,y,0) == 1:
						if self.InternalSquare(x,y+1) == 0:
							ty = 0
							for z in range(self.Height[layerN]-1):
								if ty != 0 and GetTile(x,y+ty,0) != 0:
									redo = 1
								if (x,y+ty) in self.BadTiles:
									redo = 1
								SetTile(x,y+ty,L,42)
								ty+=1	
							if (x,y+ty) in self.BadTiles:
								redo = 1
							if ty != 0 and GetTile(x,y+ty,0) != 0:
								redo = 1										
							SetTile(x,y+ty,L,42)
							
							#self.BadTiles.append((x,y+ty))
						
					#Forward-facing right edge cliff
					#If there's no cliff to the right but one to the left or bottom left. If it's bottom left, place a side right cliff next to us. Make sure the area below it isn't internal
					if GetTile(x+1,y,0) == 0 and GetTile(x,y+1,0) == 0:
						if GetTile(x-1,y,0) == 1 or GetTile(x-1,y+1,0) == 1:
							print "%s %s %s" % (x,y,self.InternalSquare(x,y+1))
							if self.InternalSquare(x,y+1) == 0:
								ty = 0
								for z in range(self.Height[layerN]-1):
									if ty != 0 and GetTile(x,y+ty,0) != 0:
										redo = 1
									if (x,y+ty) in self.BadTiles:
										redo = 1
									SetTile(x,y+ty,L,47)
									ty+=1
								if (x,y+ty) in self.BadTiles:
									redo = 1
									
								if ty != 0 and GetTile(x,y+ty,0) != 0:
									redo = 1											
								SetTile(x,y+ty,L,45)
								
								#self.BadTiles.append((x,y+ty))
								
							
					#Forward-facing left edge cliff
					#If there's no cliff to the left but one to the right or bottom right. If it's bottom left, place a side right cliff next to us. Make sure the area below it isn't internal
					if GetTile(x-1,y,0) == 0 and GetTile(x,y+1,0) == 0:
						if GetTile(x+1,y,0) == 1 or GetTile(x+1,y+1,0) == 1:
							
							if self.InternalSquare(x,y+1) == 0:
								ty = 0
								for z in range(self.Height[layerN]-1):
									if ty != 0 and GetTile(x,y+ty,0) != 0:
										redo = 1		
									if (x,y+ty) in self.BadTiles:
										redo = 1							
									SetTile(x,y+ty,L,46)
									ty+=1
								if (x,y+ty) in self.BadTiles:
									redo = 1

								if ty != 0 and GetTile(x,y+ty,0) != 0:
									redo = 1					
									
								SetTile(x,y+ty,L,44)
								
								#self.BadTiles.append((x,y+ty))

					#Top edge
					#If there's nothing above this one but there is something below it at some point that isn't directly below.
					if GetTile(x,y-1,0) == 0 and GetTile(x,y+1,0) == 0 and self.InternalSquare(x,y+1) == 1:
						SetTile(x,y,L,53)
		
		#return 1
		
		for x in range(self.x,self.x+self.w):
			for y in reversed(range(self.y,self.y+self.h)):
				if GetTile(x,y,0) == 1:						
					#Right-side cliff
					#If there are cliffs above and below this cliff and one is also right sided.
					if GetTile(x,y+1,0) == 1 and GetTile(x,y-1,0) == 1 and GetTile(x,y+1,L) in [45,47,49]:
						SetTile(x,y,L,49)
						
					#Left-side cliff
					#If there are cliffs above and below this cliff and one is also left sided.
					if GetTile(x,y+1,0) == 1 and GetTile(x,y-1,0) == 1 and GetTile(x,y+1,L) in [44,46,48]:
						SetTile(x,y,L,48)			

					#if there's something directly below and directly to the left of this
					if GetTile(x,y+1,0) == 1 and GetTile(x-1,y,0) == 1:
						#If it's a front edge to the left then we're appropriatley set as a left edge.
						if GetTile(x-1,y,L) in [45,47,44,46,42]:
							SetTile(x,y,L,48)
						else:
						#If it's not an edge, then we're just a top tile
							SetTile(x,y,L,53)
							
					#if there's something directly below and directly to the right of this
					if GetTile(x,y+1,0) == 1 and GetTile(x+1,y,0) == 1:
						#If it's a front edge to the right then we're appropriatley set as a right edge.
						if GetTile(x+1,y,L) in [45,47,44,46,42]:
							SetTile(x,y,L,49)
						else:
						#If it's not an edge, then we're just a top tile
							SetTile(x,y,L,53)

					#if there's something directly above and directly to the right of this with something below it.
					if GetTile(x,y-1,0) == 1 and GetTile(x+1,y,0) == 1 and self.InternalSquare(x,y+1) == 1:
						#Then we're a right edge
						SetTile(x,y,L,49)
						
					#if there's something directly above and directly to the left of this with something below it.
					if GetTile(x,y-1,0) == 1 and GetTile(x-1,y,0) == 1 and self.InternalSquare(x,y+1) == 1:
						#Then we're a left edge
						SetTile(x,y,L,48)
						
		if redo:
			return 0
			
		else:			
			return 1
	
	def InsideBadTiles(self,x,y):
		
		#Left
		x2,y2 = (x,y)
		L = 0
		while x2 >= self.x:
			#print str(self.GetTile(x2,y2,0))
			if (x2,y2) in self.BadTiles:
				L = 1
			x2-=1
			
		#Right 
		x2,y2 = (x,y)
		R = 0
		while x2 < self.x+self.w:
			#print str(self.GetTile(x2,y2,0))
			if (x2,y2) in self.BadTiles:
				R = 1
			x2+=1
			
		#Up
		x2,y2 = (x,y)
		U = 0
		while y2 >= self.y:
			#print str(self.GetTile(x2,y2,0))
			if (x2,y2) in self.BadTiles:
				U = 1
			y2-=1
			
		#Down
		x2,y2 = (x,y)
		D = 0
		while y2 < self.y+self.h:
			#print str(self.GetTile(x2,y2,0))
			if (x2,y2) in self.BadTiles:
				D = 1
			y2+=1
			
		if L and R and D and U:
			return 1
		else:
			return 0		
	
	def InternalSquare(self,x,y):
		#Check to see whether the tile is INSIDE (1) the cliff or OUTSIDE (0).
		#This is done by scanning continually in all 4 directions. If a wall is found in all 4, then it is an internal cliff.
		
		#Left
		x2,y2 = (x,y)
		L = 0
		while x2 >= self.x:
			#print str(self.GetTile(x2,y2,0))
			if self.GetTile(x2,y2,0) != 0:
				L = 1
			x2-=1
			
		#Right 
		x2,y2 = (x,y)
		R = 0
		while x2 < self.x+self.w:
			#print str(self.GetTile(x2,y2,0))
			if self.GetTile(x2,y2,0) != 0:
				R = 1
			x2+=1
			
		#Up
		x2,y2 = (x,y)
		U = 0
		while y2 >= self.y:
			#print str(self.GetTile(x2,y2,0))
			if self.GetTile(x2,y2,0) != 0:
				U = 1
			y2-=1
			
		#Down
		x2,y2 = (x,y)
		D = 0
		while y2 < self.y+self.h:
			#print str(self.GetTile(x2,y2,0))
			if self.GetTile(x2,y2,0) != 0:
				D = 1
			y2+=1
			
		if L and R and D and U:
			return 1
		else:
			return 0
	
	def FindEdge(self,x,y,dirX,dirY):
		GetTile = self.GetTile
		
		while GetTile(x,y,0) != 0:
			if GetTile(x,y,1) in [44,46,45,47]:
				return GetTile(x,y,1)
			
			x += dirX
			y += dirY
		
		return 0
	
	def GetTile(self,x,y,l):
		
		w = ika.Map.GetLayerProperties(l)[1]
		h = ika.Map.GetLayerProperties(l)[2]
		
		if x >= 0 and y >= 0 and x <= w and y <= h:
			return ika.Map.GetTile(x,y,l)
		else:
			return 0
		
	def SetTile(self,x,y,l,t):
		ika.Map.SetTile(x,y,l,t)
		
	def TilesBelow(self,x,y):
		y+=1
		
		while y < self.y+self.h:
			if self.GetTile(x,y,0) != 0:
				return 1
			
			y+=1
			
		return 0
		
	def ApplyShadows(self):
		
		return 1
		
		last_x = 0
		edge = 0
		n = 0
		
		for y in range(self.y,self.y+self.h+self.Height):
			for x in range(self.x,self.x+self.w):
				if self.GetTile(x,y,1) != 68 and self.GetTile(x,y,1) != 53:
					last_x = x
					edge = 1
					
			if edge:
				n+=1
				if n > 1:
					self.SetTile(last_x+1,y,3,73)
			edge = 0
		
def SortX(a,b):
	
	if a.x > b.x:
		return 1
	elif a.x < b.x:
		return -1
	elif a.x == b.x:
		return 0

def SortY(a,b):
	
	if a.y > b.y:
		return 1
	elif a.y < b.y:
		return -1
	elif a.y == b.y:
		return 0	


def GenerateLevel1():

	for x in range(40):
		for y in range(45):
			ika.Map.SetTile(x,y,0,0)
			ika.Map.SetTile(x,y,1,68)
			ika.Map.SetTile(x,y,2,68)
			ika.Map.SetTile(x,y,3,68)
			ika.Map.SetTile(x,y,4,68)

	c1 = Cliff(10,10,25,25,
			150, #Attempts
			1, #Layers
			[1], #Height
			0, #Caves
			0) #Ladders

	c1.Generate()


	x = 0
	y = 0
	count = 0
	
	while count < 10:
		x = ika.Random(0,40)
		y = ika.Random(0,45)
		w = ika.Random(5,20)
		h = ika.Random(5,20)
		
		if not GetAreaObs(x,y,w,h+2):
			c1 = Cliff(x,y,w,h,
					30, #Attempts
					1, #Layers
					[ika.Random(1,3)], #Height
					0, #Caves
					0) #Ladders

			c1.Generate()
			
			count +=1
	


def GenerateMap():
	
	#(x,y,w,h,points,layers,height,caves,ladders)
	
	GenerateLevel1()
	
	#return 1
	
	#c1 = Cliff(0,0,10,10,
	#		15, #Points 
	#		2, #Layers
	#		[2,1,1,1], #Height
	#		0, #Caves
	#		0) #Ladders
			
	#c2 = Cliff(6,0,9,9,
	#		15, #Points 
	#		2, #Layers
	#		[1,1], #Height
	#		0, #Caves
	#		0) #Ladders

	#c1.Generate()
	#c2.Generate()
	

	
	#for x in range(20):
	#	for y in range(20):
	#		ika.Map.SetTile(x,y,0,67)
			
	#for z in c1.BadTiles:
	#	x,y = z
	#	ika.Map.SetTile(x,y,0,67)


def GetAreaObs(x1,y1,w,h):
	
	for x in range(x1,x1+w):
		for y in range(y1, y1+h):
			if ika.Map.GetTile(x,y,1) != 68 or ika.Map.GetTile(x,y,2) != 68:
				return 1
	return 0
	
def GetAreaInternalObs(x1,y1,w,h):
	
	for x in range(x1,x1+w):
		for y in range(y1, y1+h):
			if ika.Map.GetTile(x,y,1) != 68 or ika.Map.GetTile(x,y,2) != 68:
				return 1
	return 0

def DrawLine(p1, p2):
	
	p1 = [p1.x, p1.y]
	p2 = [p2.x, p2.y]
	
	for x,y in Brensenham_line(p1, p2):
		ika.Map.SetTile(x,y,0,1)
	
def CheckLineObs(p1,p2):
	
	p1 = [p1.x, p1.y]
	p2 = [p2.x, p2.y]	
	
	obsC = 0
	
	for x,y in Brensenham_line(p1, p2):
		if ika.Map.GetTile(x,y,0) == 1:
			obsC+=1
			
	if obsC > 1:
		return True
	else:
		return False
			
	