import engine
import ika
import frames
import data
import animationscript
import transition
import queue
import textbox
from controls import *
import ai
import zone

class BattleManager:
	
	def __init__(self, characters):
		
		self.Entities = [ ]
		
		self.Console = Console()
		self.OrderBar = OrderBar(self)
		self.TotalXP = 0
		
		Letters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
		
		#	(self, x, y, character, letter, parent)
		i=0
		
		s = 900
		self.Reset = 0
		for x in characters:
			if x.Hero == 0:
				e = Enemy(x.x, x.y, x, Letters[i], self)
				e.Character.Name = e.Character.Name+" "+Letters[i]
				self.Entities.append(  e) 
				s-=ika.Random(50,100)
				e.Charge = s
				
				self.TotalXP += e.Character.XP
				i+=1
				
				self.Console.AddLine("%s appears!" % (e.Character.Name) )
			else:
				p = Player(x.x, x.y, x, 'P', self)
				p.Charge = 900-ika.Random(50,150)
				self.Entities.append( p )
		
		player = self.GetPlayer()
		
		for x in characters:
			if x.Hero == 0:
				x.Target = player
		
		player.Target = self.GetNextEnemy(0)
		self.Player = self.GetPlayer()
		
		self.Done = 0
		
		engine.GGetPlayer().Freeze()

	
	def Render(self):
		
		for x in self.Entities:
			
			x.Render()
			
		if self.Player.Target != None:
			engine.Images["Pointer"].Blit(self.Player.Target.x+(self.Player.Target.Character.Sprite.BarY/2), self.Player.Target.y+self.Player.Target.Character.Sprite.BarY+10)
			
		self.OrderBar.Render()
		self.Console.Render()
		
	def Update(self):
		
		for x in self.Entities: 
			x.Update()
			
		for i, x in enumerate(self.Entities):	
			if hasattr(x,"Dead") and x.Dead:
				self.Entities.pop(i)
				
		if len(self.Entities) == 1 and not self.Done:
			
			for x in self.GetPlayer().StatusEffects:
				x.LastExecute(self.GetPlayer())

			engine.Queue.Add( transition.TransitionTo(ika.RGB(255,255,255,0) ) )
			engine.Queue.Add( queue.Function(StopCombat) )
			
			levels = engine.GGetPlayer().AddExp(self.TotalXP)
			engine.Queue.Add( queue.Function(engine.GGetPlayer().EnableBar), 0)
			if zone.CurrentArea in ["1","2","3","4","5"]:
				engine.Queue.Add( queue.Function( engine.PlayMusic, "Desert") ,0)
			else:
				engine.Queue.Add( queue.Function( engine.PlayMusic, "Cave") ,0)
			engine.Queue.Add( transition.TransitionFrom(ika.RGB(255,255,255,0) ),0 )
			
			if levels == 0:
				engine.Queue.Add( textbox.TextBox("Marcus gained %s experience!" % (self.TotalXP)),1)
			elif levels == 1:
				engine.Queue.Add( textbox.TextBox("Marcus gained %s experience!\n\nGained a level!" % (self.TotalXP)),1 )
			elif levels == 2:
				engine.Queue.Add( textbox.TextBox("Marcus gained %s experience!\n\nGained %s levels!" % (self.TotalXP,levels)),1 )

			if levels >= 1:
				engine.Queue.Add(queue.Function(engine.GGetPlayer().ProcessXP))
			engine.Queue.Add(queue.Function(engine.GGetPlayer().UnFreeze))
			self.Done = 1
			
		self.Console.Update()
		
		if self.GetPlayer().Character.HP <= 0 and self.Reset == 0:
			self.Reset = 1
			engine.GGetPlayer().ResetGame()
		
	def GetAllEnemy(self):
		
		List = []
		
		for x in self.Entities:
			if not x.Player:
				List.append(x)
				
		return List

	def GetNextEnemy(self, enemy):

		all = self.GetAllEnemy()
		next = 0
		
		if enemy == 0:
			return all[0]

		for x in all+all:
			#print str(x)
			if next == 1:
				return x
			
			if x is enemy:
				#print 'next!'
				next = 1
				
	def GetPreviousEnemy(self, enemy):

		all = self.GetAllEnemy()
		next = 0

		previous = None
		
		if enemy == 0:
			return all[0]	

		for x in all+all:
			
			if x is enemy and previous is not None:
				return previous
				
			previous = x
			
	def GetPlayer(self):
		
		for x in self.Entities:
			if x.Character.Hero:
				return x
		
	def GetRandomEntity(self):

		n = ika.Random(0, len(self.Entities))
		
		return self.Entities[n]
		
class Console:
	#List of battle information text.
	
	Color = ika.RGB(0,0,0,255)
	
	def __init__(self):
		
		self.Buffer = []
		self.CurrentPoint = 0
		self.CurrentLine = 0
		
	def AddLine(self, text):
		self.Buffer.append(text)
		
		if len(self.Buffer) > 5:
			
			self.Buffer.pop(0)
	
	def Render(self):
		engine.Windows["Menu"].Blit(5,5, 310, 50,self.Color)
		
		i = 0
		
		for x in self.Buffer:
			engine.Fonts["Interface"].Print(5, 5+(i*10), x)
			i+=1
	
	def Update(self):
		pass
	
class CooldownBar:	
	#Skill cooldown bar
	Active = ika.RGB(0,220,220,255)
	Background = ika.RGB(0,20,0,255)
	Border = ika.RGB(0,255,0,255)
	
	Counter = ika.Image("images\\counter.png")
	Guard = ika.Image("images\\guard.png")
	
	w = 30
	h = 5
	
	def Render(self, x, y, cooldown, state):
		
		#engine.Fonts["Interface"].Print(20, 220, str(cooldown))
		
		if cooldown > 1:
			
			#engine.Fonts["Interface"].Print(20, 200, str(cooldown))
			#engine.Fonts["Interface"].Print(20, 220, str(int(x+(self.w*(cooldown/900.)))))
		
			#ika.Video.DrawRect(x, y, x+self.w, y+self.h, self.Background,1)
			engine.Windows["Menu"].Blit(x, y, self.w, self.h,self.Border)
			ika.Video.DrawRect(x-2, y-2, x+int((self.w*(cooldown/900.)))-2, y+self.h+1, self.Active,1)
			
			if state == 2:
				self.Guard.Blit(x+12,y-1)
				
			if state == 3:
				self.Counter.Blit(x+10,y-3)
			
			#ika.Video.DrawRect(x, y, x+50,y+20, self.Color,1)
			#ika.Video.DrawRect(x, y, x+self.w, y+self.h, self.Border,0)		
			
			
	
class OrderBar: 
	#Order of turns bar
	
	def __init__(self, parent):
		
		self.Parent = parent
		self.Color = ika.RGB(0,0,255,255)
		#self.Image = ika.Image("images\\orderbar.png")
		
		
	def Render(self):
		
		#self.Image.Blit(10,220)
		engine.Windows["Menu"].Blit(5,62, 310, 5,self.Color)
		
		for x in self.Parent.Entities:
			engine.Fonts["Order"].Print(3+x.Charge/3.0,60,x.Letter)
			
	
def CalculateAttack(character, skill, formula):	
	
	if formula == 0:
		return skill.Power
	elif formula == 1:
		return (character.Strength*2)+ika.Random(character.Luck/2,character.Luck)
	elif formula == 2:
		return (character.Strength/2)+ika.Random(character.Luck/2,character.Luck)
	elif formula == 3:
		return character.Strength+ika.Random(character.Luck/2,character.Luck)
	elif formula == 4:
		return 0
		#return character.Intelligence
	elif formula == 5:
		return 0
		#return character.Intelligence*2
	elif formula == 6:
		return 0
		#return character.Intelligence/2
	elif formula == 7:
		return 0
		#return ika.Random(character.Intelligence, character.Intelligence*1.5)
	
class StatusEffect:
	
	def __init__(self, data, caster):
		
		self.Name = data.Name
		self.Data = data
		self.Turns = data.Turns
		self.Dead = 0
		self.FullEffect = 0
		
		self.CasterEntity = caster
		self.Caster = caster.Character
		
		self.Image = ika.Image(str("images\\icons\\"+data.Image))
		
		self.BattleEffect = None

	def FirstExecute(self, entity):
				
		entity.Character.Strength += self.Data.Strength
		entity.Character.Defense += self.Data.Defense
		entity.Character.Speed += self.Data.Speed
		
		if self.Data.Effect == 1:
			entity.Confused = 1
		
		if self.Data.BattleEffect is not None:
			self.BattleEffect = BattleEffect(self.Data.BattleEffect)
		
	def TurnExecute(self, entity):
		
		if self.Data.Effect == 2: #Poison
			amt = entity.HPDamage(CalculateAttack(self.Caster, self, self.Data.PowerFormula), self.CasterEntity )

			entity.Console.AddLine(self.Data.ProcessString % {"self": entity.Character.Name, "amt": amt } )
				
			
		self.Turns-=1
		if self.Turns <= 0:
			self.LastExecute(entity)
			self.Dead = 1

	def LastExecute(self, entity):
		
		entity.Console.AddLine(self.Data.EndString % {"self": entity.Character.Name} )
		
		if self.Data.Effect == 1:
			entity.Confused = 0
		
		entity.Character.Strength -= self.Data.Strength
		entity.Character.Defense -= self.Data.Defense
		entity.Character.Speed -= self.Data.Speed
		
	def Render(self,x,y):
		
		if self.BattleEffect is not None:
			self.BattleEffect.Render(x,y)
			
		self.Image.Blit(x,y)
		
	def Update(self):
		if self.BattleEffect is not None:
			self.BattleEffect.Update()
	
class BattleEffect:
	#Visual effect for attacks/status effects
	
	def __init__(self, data):
		
		self.Frame = 0
		self.Script = animationscript.AnimationScript(self)
		self.Script.SetScript(data.AnimationScript)
		#(x,y,img, width, height, span,xdist,ydist, numtiles)
		self.ImageList = frames.LoadFrames(0,0,"images\\effects\\"+str(data.Image), data.Width, data.Height, data.FrameNumber,0,0,data.FrameNumber)
	
	def Render(self, x, y):
		
		self.ImageList[self.Frame].Blit(x,y)
		
	def Update(self):
		
		self.Script.Update()
		
		if self.Script.Finished:
			
			self.Dead = 1

Offsets = [0,0,0,0,0,0,0,0,1,2,3,4,5,7,9,10,11,11,11,11,11,11,11,11,10,8,6,4,2,1,2,3,4,5,6,7,8,7,6,5,4,3,2,1,2,3,4,5,5,5,4,3,2,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0] 


class NumberEffect:
	#Visual effect for numbers
	
	def __init__(self, x,y, number, color):
		
		self.Number = str(number)
		self.Position = 0
		self.Timer = 0
		
		self.Color = color #red or blue
		
		self.x=x
		self.y=y
	
	def Render(self,x,y):
		
		for i,dmg in enumerate(self.Number):
			n = self.Position+(i*4)
			if n > len(Offsets)-1:
				n = len(Offsets)-1
				
			#xy = utility.MapToScreen([self.x,self.y])
			
			#NOTE: Use images later to add a fade effect.
			engine.Fonts["Interface"].Print(self.x+(i*8),self.y-Offsets[n],self.Color+dmg)
		
	def Update(self):
		
		self.Timer += 1
		
		if self.Timer > 1:
		
			self.Position += 1
			
			if self.Position-5 >= len(Offsets)-1:
				self.Dead = 1
				
			self.Timer = 0
	
class BattleSprite:
	
	def __init__(self, data):
		
		self.x = 0
		self.y = 0
		
		self.Data = data

		self.Script = animationscript.AnimationScript(self)
		self.Color = ika.RGB(255,255,255,255)
		#print data.Image
		self.Image = ika.Image('images\\monsters\\'+str(data.Image))
		
		self.Script.SetScript(data.IdleScript)
		
	def Render(self, x, y):
		
		ika.Video.TintBlit(self.Image,x+self.x, y+self.y,self.Color)
		#engine.Fonts["Interface"].Print(0,100,str(self.x))
		
	def Update(self):
		
		self.Script.Update()
	
class BattleEntity:
	
	def __init__(self, x, y, character, letter, parent):
		
		self.x = x
		self.y = y
		
		self.Letter = letter
		
		self.Dying = 0
		self.Dead = 0
		self.Confused = 0
		
		self.Cooldown = 0 #Skill cool down OR counter/defense length.
		self.Charge = 900 #Charge up to have a turn. MAX 900
		self.TurnWait = 0 #Whether waiting to perform your turn or not.
		self.QueuedSkill = None #Which skill is being charged. 
		self.State = 0 #0 - normal 1 - open 2 - guard 3 - counter
		
		self.Parent = parent
		self.Console = parent.Console
		
		self.CooldownBar = CooldownBar()
		
		self.Character = character #Stat/hp container
		if not character.Hero:
			self.Sprite = BattleSprite(self.Character.Sprite)
		self.StatusEffects = [ ]
		self.Effects = []
		
		
		
		self.Target = 0
		
	def Render(self):
		
		if not self.Character.Hero:
			self.Sprite.Render(self.x, self.y)
			
			engine.Fonts["Order"].Print(self.x+self.Sprite.Data.BarX-12, self.y+self.Sprite.Data.BarY, self.Letter)

		if not self.Character.Hero:
			self.CooldownBar.Render(self.x+self.Sprite.Data.BarX,self.y+self.Sprite.Data.BarY,self.Cooldown,self.State)
			
		if not self.Character.Hero:	
			
			z = 0
			for x in self.StatusEffects:
				x.Render(self.x+z+10, self.y+10)
				z+= 25
				
		else:
			z = 0
			for x in self.StatusEffects:
				x.Render(90+z, 72)
				z+= 25
		
		for x in self.Effects:
			if not isinstance(x, NumberEffect):
				x.Render(self.x, self.y)

		for x in self.Effects:
			if isinstance(x, NumberEffect):
				x.Render(self.x, self.y)

		
	def Update(self):
		
		
		
		if not self.Character.Hero:
			self.Sprite.Update()
			
		if self.Dying and not self.Character.Hero and self.Sprite.Script.Finished:
			self.Console.AddLine("%s was defeated!" % (self.Character.Name) )
			self.Dead = 1
			
		if self.Dying:
			return 0
		
		if (self.Cooldown == 0 and self.Charge == 0 and self.TurnWait == 0): #Is it the entity's turn?
			self.OnTurn()
			self.TurnWait = 1
			
		elif self.TurnWait == 1 and self.Cooldown == 0: #It has been the entity's turn, now to wait for a skill choice.
			
			if self.QueuedSkill != None:
				self.Cooldown = self.QueuedSkill.Cooldown
				self.TurnWait = 2
				
				if self.QueuedSkill.Effect == 3 or self.QueuedSkill.Effect == 4:
					self.OnAttack()
				
		elif self.Cooldown != 0 and self.TurnWait == 2: #A skill is cooling down.
			
			self.Cooldown -= 1
			
			if self.Cooldown <= 0: #The cooldown is over, execute skill
				if self.QueuedSkill.Effect != 3 and self.QueuedSkill.Effect != 4:
					self.OnAttack()
					
				self.Charge = 900
				self.Cooldown = 0
				self.TurnWait = 0
				self.QueuedSkill = None
				self.State = 0
			
		elif self.Charge != 0: #if we have some turn charging to do, then do so.
			
			if self.Character.Speed <= 0:
				self.Charge -= 1
			else:
				self.Charge -= self.Character.Speed
			
			if self.Charge <= 0:
				self.Charge = 0
				
		for x in self.Effects:
			x.Update()
			
		for x in self.StatusEffects:
			x.Update()			
			
		for i,x in enumerate(self.Effects):	
			if hasattr(x,"Dead") and x.Dead:
				self.Effects.pop(i)
				

		
	def OnTurn(self):
		#Process status effects
		for x in self.StatusEffects:
			x.TurnExecute(self)
			
		for i,x in enumerate(self.StatusEffects):	
			if hasattr(x,"Dead") and x.Dead:
				self.StatusEffects.pop(i)

	def HPDamage(self, amt, other, absolute=0):
		
		if (amt > 0):
			if not absolute:
				damage = amt - ika.Random(self.Character.Defense/1.5,self.Character.Defense)
			else:
				damage = amt
				
			if damage < 0 or self.State == 2  or self.State == 3:
				damage = 0
				
			if self.State == 3:
				#print str(self)
				#print str(other)
				self.OnAttack(self.Character.Skills[0], other)
			
			if self.Character.Hero == 0:
				self.Effects.append( NumberEffect(self.x+self.Sprite.Data.DamageX+ika.Random(-40,40), self.y+self.Sprite.Data.DamageY, damage, "#[FF0000FF]") )
			else:
				self.Effects.append( NumberEffect(self.x+20+ika.Random(-40,40), self.y+40, damage, "#[FF0000FF]") )
		else:
			damage = amt
			
			if self.Character.Hero == 0:
				self.Effects.append( NumberEffect(self.x+self.Sprite.Data.DamageX+ika.Random(-40,40), self.y+self.Sprite.Data.DamageY, -damage, "#[FF3FFF00]") )
			else:
				self.Effects.append( NumberEffect(self.x+40+ika.Random(-40,40), self.y+40, -damage, "#[FF3FFF00]") )
			
		self.Character.HP -= damage
		
		if (self.Character.HP > self.Character.Vitality):
			self.Character.HP = self.Character.Vitality
		if self.Character.HP <= 0:
			self.Dying = 1
			if not self.Character.Hero:
				self.Sprite.Script.SetScript("rgba255,255,255,255,255,0,0,0,4")
			
		if damage >= 0:	
			return damage
		else:
			return -damage
		
	def HydrationDamage(self, amt):
		
		self.Character.Hydration -= amt
		
		if self.Character.Hero == 1:
			if amt < 0:
				self.Effects.append( NumberEffect(self.x, self.y+40, -amt, "#[FFFFF600]") )
			else:
				if amt != 0:
					self.Effects.append( NumberEffect(self.x, self.y+40, amt, "") )
		
		return amt
		
	def AddStatusEffect(self, effect, other):
		
		if self.State == 2 or self.State == 3:
			return 0
		
		for x in self.StatusEffects:
			
			if x.Data.Name == effect.Name:
				return 0
				
		e = StatusEffect(effect,other)
		self.StatusEffects.append(e)
		e.FirstExecute(self)
		
		return 1
		
	def Cancel(self, amt):
		
		if self.TurnWait == 2:
			self.Cooldown = 0
			self.State = 0
			self.Charge = 900
			self.TurnWait = 0
			
			return 1
			
		return 0
			
	def AddBattleEffect(self, effect):
		
		self.Effects.append(BattleEffect(effect))

	def OnAttack(self, skill=None, target=None):
		
		if (skill is None):
			Skill = self.QueuedSkill
		else:
			Skill = skill
		
		#---------------------------------------
		#GENERATE TARGET LIST
		#---------------------------------------
		
		if self.Character.Hero:
		
			if target is not None:
				Targets = [target]
		
			elif (Skill.Target == 0 or Skill.Target == 4): #single enemy or both
				Targets = [self.Target]
				
			elif (Skill.Target == 1): #all enemy
				Targets = self.Parent.GetAllEnemy()
				
			elif Skill.Target == 2: #player
				
				Targets = [self]
				
			elif Skill.Target == 3 or self.Confused: #random
				
				Targets = [self.Parent.GetRandomEntity()]
				
		else:
			if target is not None:
				Targets = [target]			
			
			elif (Skill.Target == 0 or Skill.Target == 4): 
				Targets = [self.Parent.GetPlayer()]

			elif (Skill.Target == 1): #all enemy
				Targets = self.Parent.GetAllEnemies()
				
			elif Skill.Target == 2: #self
				
				Targets = [self]
			
			if Skill.Target == 3 or self.Confused: #random
				
				Targets = [self.Parent.GetRandomEntity()]		
					
		#---------------------------------------
		#GENERATE POWER AMOUNT
		#---------------------------------------
		
		Power = 0
		Power2 = 0
		Success = -1
		
		Power = CalculateAttack(self.Character, Skill, Skill.PowerFormula)
		
		#---------------------------------------
		#ATTACK PROCESSING
		#---------------------------------------
		
		if skill is None:
			self.State = 1
		
		for t in Targets:
				
			if Skill.Effect == 1: #Damage
				Power = t.HPDamage(Power, self)
			elif Skill.Effect == 2: #Cancel
				Success = t.Cancel(Power)
			elif Skill.Effect == 3: #Counter
				self.State = 3
				Success = 1
			elif Skill.Effect == 4: #Guard
				self.State = 2
				Success = 1
			elif Skill.Effect == 5: #Heal
				Power = t.HPDamage(-Power, self)
			elif Skill.Effect == 6: #Drain
				if self.Character.Hero:
					Power = t.HPDamage(Power, self, absolute=1)
					Power = -self.HydrationDamage(-Power)
				else:
					t.HydrationDamage(Power)
			elif Skill.Effect == 7: #Hydration Damage
				Power = t.HydrationDamage(Power)
			elif Skill.Effect == 8:
				Power = t.HPDamage(Power, self, absolute=1)
				
			if Skill.Causes != None:
				Success = t.AddStatusEffect(Skill.Causes, self)
				
			if Skill.BattleEffect != None and Success != 0:
				t.AddBattleEffect(Skill.BattleEffect)
				

		#---------------------------------------
		#FINAL PROCESSING
		#---------------------------------------

		if skill is None:

			if (Skill.HP_Cost > 0):
				self.HPDamage(Skill.HP_Cost,self)
			self.HydrationDamage(Skill.Hydration_Cost)
			
			self.Cooldown = Skill.Cooldown
		
		if Success == -1:
			for t in Targets:
				self.Console.AddLine(Skill.AttackString % {"self": self.Character.Name, "target": t.Character.Name, "amt": Power, "amt2": Power2} )
			
		else:	
			if Success == 0:
				self.Console.AddLine(Skill.FailString % {"self": self.Character.Name, "target":  Targets[0].Character.Name} )
			else:
				self.Console.AddLine(Skill.SuccessString % {"self": self.Character.Name, "target":  Targets[0].Character.Name} )
			
class Enemy(BattleEntity):
	
	Player = 0
	
	def OnTurn(self):
		BattleEntity.OnTurn(self)

		allies = self.Parent.GetAllEnemy()
		allies.remove(self)
		player = self.Parent.GetPlayer()

		#Process AI
		#self.QueuedSkill = self.Character.Skills[0]
		
		self.QueuedSkill = self.Character.Skills[ai.ProcessAI(self,allies,player)]
		
		self.Console.AddLine(self.QueuedSkill.ChargeString % {"self": self.Character.Name} )
	
class Player(BattleEntity):
	
	Color = ika.RGB(0,0,0,255)
	Bar = ika.RGB(205,0,0,255)
	Player = 1
	
	w = 80
	h = 8
	
	def Render(self):
		
		skill = 0
		if self.QueuedSkill != None:
			skill = 1
		
		self.CooldownBar.Render(45,92,self.Cooldown,self.State)	
		#Draw HP, Hydration, Skills, Attack

		#HP bar
		engine.Windows["Menu"].Blit(5,75, self.w, self.h,self.Color)
		ika.Video.DrawRect(5-1, 75-1, 5+int((self.w*(self.Character.HP/(self.Character.Vitality*1.0)))), 75+self.h, self.Bar,1)
		engine.Fonts["Interface"].Print(5,75, "%s / %s" % (self.Character.HP, self.Character.Vitality) )

		#Hydration count
		engine.Windows["Menu"].Blit(5,90, 25, self.h,self.Color)
		engine.Fonts["Interface"].Print(5, 90, "%s" % (self.Character.Hydration) )
		
		
		#engine.Windows["Menu"].Blit(130, 200, 60, 10,self.Color)
		
		#engine.Windows["Menu"].Blit(130, 215, 60, 17,self.Color)
		
		#engine.Windows["Menu"].Blit(130-70, 215, 60, 17,self.Color)
		
		#engine.Windows["Menu"].Blit(130+70, 215, 60, 17,self.Color)
		
		for x in range(2,5):
			
			if skill and self.Character.Skills[x] == self.QueuedSkill:
				engine.Windows["Select"].Blit(60+(70*(x-2)), 217, 60, 6,self.Color)
			else:
				engine.Windows["Menu"].Blit(60+(70*(x-2)), 217, 60, 6,self.Color)
		
			engine.Fonts["Interface"].CenterPrint(90+(70*(x-2)), 216, self.Character.Skills[x].Name+" #[FF0000FF]%s #[FFF6FF00]%s" % (self.Character.Skills[x].HP_Cost,self.Character.Skills[x].Hydration_Cost))
			#engine.Fonts["Interface"].CenterPrint(90+(70*(x-2)), 223,  )
		
		if skill and self.Character.Skills[0] == self.QueuedSkill:
			engine.Windows["Select"].Blit(130, 203, 60, 6,self.Color)
		else:
			engine.Windows["Menu"].Blit(130, 203, 60, 6,self.Color)
		engine.Fonts["Interface"].CenterPrint(160,202,"Attack")
		
		if skill and self.Character.Skills[1] == self.QueuedSkill:
			engine.Windows["Select"].Blit(130, 230, 60, 6,self.Color)
		else:
			engine.Windows["Menu"].Blit(130, 230, 60, 6,self.Color)
		engine.Fonts["Interface"].CenterPrint(160,229,"Guard")
	
		BattleEntity.Render(self)
	
	
		
	
	def Update(self):
		
		#Input code
		BattleEntity.Update(self)
		
		if self.Target != None:
			if self.Target.Dying:
				self.Target = self.Parent.GetNextEnemy(self.Target)
		
			position, pressed= GetButtons()
			
			if self.Cooldown == 0:
			
				if "attack" in pressed:
					self.QueuedSkill = self.Character.Skills[0]
				elif "guard" in pressed:
					self.QueuedSkill = self.Character.Skills[1]			
				elif "skill_a" in pressed:
					self.QueuedSkill = self.Character.Skills[2]
				elif "skill_b" in pressed:
					self.QueuedSkill = self.Character.Skills[3]
				elif "skill_c" in pressed:
					self.QueuedSkill = self.Character.Skills[4]

			if "right" in pressed:
				self.Target = self.Parent.GetNextEnemy(self.Target)
			if "left" in pressed:
				self.Target = self.Parent.GetPreviousEnemy(self.Target)

		
		
class Character:
	
	def __init__(self, x, y, data):
		
		self.x = x
		self.y = y
		
		self.Hero = 0
		
		self.Name = data.Name
		#self.AttackDescription = data.AttackDescription
		
		self.HP = int(data.Vitality)
		self.Hydration = 0
		
		self.Vitality = data.Vitality
		self.Strength = data.Strength
		self.Defense = data.Defense
		self.XP = data.XP
		self.Speed = data.Speed
		self.Luck = data.Luck
		
		self.Skills = data.Skills
		
		self.AI = data.AI
		
		self.Sprite = data.MonsterSprite
		
		self.id = data.id
		
		self.Ai1 = 0
		self.Ai2 = 0
		self.Ai3 = 0
		
class HeroCharacter(Character):
	
	def __init__(self, hp, hyd, vit, str, defen,speed,lck, xp, level, skills):
		
		self.x = 4
		self.y = 74
		
		self.Hero = 1
		
		self.Name = "Marcus"
		#self.AttackDescription = "Marcus struck the %s for %s!"
		
		self.HP = hp
		self.Hydration = hyd
		
		self.Vitality = vit
		self.Strength = str
		self.Defense = defen
		self.Speed = speed
		self.Luck = lck
		
		self.XP = xp
		self.Level = level
		
		self.Skills = skills
		
		self.AI = 0
		
def GenerateParty(data):
	
	list = []
	i = 1
	for e in data.Enemies:
		x = getattr( data, "x"+str(i) )
		y = getattr( data, "y"+str(i) )
		list.append( Character(x,y,e) ) 
		
		i+=1
		
	return list
		
def StartCombat(EnemyGroup):
	
	engine.Queue.Add( queue.Function( engine.PlayMusic, "Battle") ,0)
	engine.Queue.Add( transition.TransitionTo(ika.RGB(255,255,255,0) ) )
	
	engine.Queue.Add( queue.Function(_StartCombat, EnemyGroup),0 )
	engine.Queue.Add( queue.Function(engine.GGetPlayer().DisableBar), 0)
	
	
	engine.Queue.Add( transition.TransitionFrom(ika.RGB(255,255,255,0) ) )
	
def _StartCombat(EnemyGroup):
	
	global BM
	
	Hero = engine.GGetPlayer().Character
	Enemies = GenerateParty(data.EnemyGroup.objects.get(EnemyGroup))
	
	Entities = [Hero]+Enemies
	
	BM = BattleManager(Entities)
	
	engine.manager.UpdateList.append(BM.Update)
	engine.manager.RenderList.append(BM.Render)
	
def StopCombat():
	global BM
	
	for i,x in enumerate(engine.manager.UpdateList):
		if x == BM.Update:
			engine.manager.UpdateList.pop(i)
			
	for i,x in enumerate(engine.manager.RenderList):
		if x == BM.Render:
			engine.manager.RenderList.pop(i)		

	#engine.manager.UpdateList.append(engine.UpdateEntities)
			