import ika

def GetButtons():
    """
        Returns buttons filtered by position and pressed.
    """
    return filter(PositionFilter, buttons), filter(PressedFilter, buttons)

def PositionFilter(button):
    return buttons[button].Position() > 0.5

def PressedFilter(button):
    return buttons[button].Pressed()

deadzone = 0.5
controls = dict()
key_names = ['BACKSPACE', 'TAB', 'CLEAR', 'RETURN', 'PAUSE', 'ESCAPE',
                'SPACE', 'EXCLAIM', 'QUOTEDBL', 'HASH', 'DOLLAR', 'AMPERSAND',
                'QUOTE', 'LEFTPAREN', 'RIGHTPAREN', 'ASTERISK', 'PLUS',
                'COMMA', 'MINUS', 'PERIOD', 'SLASH', '0', '1', '2', '3', '4',
                '5', '6', '7', '8', '9', 'COLON', 'SEMICOLON', 'LESS',
                'EQUALS', 'GREATER', 'QUESTION', 'AT', 'LEFTBRACKET',
                'BACKSLASH', 'RIGHTBRACKET', 'CARET', 'UNDERSCORE',
                'BACKQUOTE', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J',
                'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V',
                'W', 'X', 'Y', 'Z', 'DELETE', 'KP0', 'KP1', 'KP2', 'KP3',
                'KP4', 'KP5', 'KP6', 'KP7', 'KP8', 'KP9', 'KP_PERIOD',
                'KP_DIVIDE', 'KP_MULTIPLY', 'KP_MINUS', 'KP_PLUS', 'KP_ENTER',
                'KP_EQUALS', 'UP', 'DOWN', 'RIGHT', 'LEFT', 'INSERT', 'HOME',
                'END', 'PAGEUP', 'PAGEDOWN', 'F1', 'F2', 'F3', 'F4', 'F5',
                'F6', 'F7', 'F8', 'F9', 'F10', 'F11', 'F12', 'F13', 'F14',
                'F15', 'NUMLOCK', 'CAPSLOCK', 'SCROLLOCK', 'RSHIFT', 'LSHIFT',
                'RCTRL', 'LCTRL', 'RALT', 'LALT', 'RMETA', 'LMETA', 'LSUPER',
                'RSUPER', 'MODE']

#Load up keyboard controls.
for key in key_names:
    controls["key:" + key] = ika.Input.keyboard[key]

# joystick:
for joyIndex, joy in enumerate(ika.Input.joysticks):
    # joystick axes:
    for axisIndex, axis in enumerate(joy.axes):
        controls['joy:%i:axis:%i+' % (joyIndex, axisIndex)] = axis
    for axisIndex, axis in enumerate(joy.reverseaxes):
        controls['joy:%i:axis:%i-' % (joyIndex, axisIndex)] = axis

    # joystick buttons:
    for buttonIndex, button in enumerate(joy.buttons):
        controls['joy:%i:buttons:%i' % (joyIndex, buttonIndex)] = button


class Button(object):
    def __init__(self, *keys):
        self.pressed = False
        self.keys = []
        for key in keys:
            self.Add(key)

    def Add(self, key):
        if key in controls:
            if controls[key] not in self.keys:
                self.keys.append(controls[key])

    def Position(self):
        values = []
        for k in self.keys:
            values.append(k.Position())
        return max(values)

    def Pressed(self):
        if not self.pressed:
            if self.Position() >= deadzone:
                self.pressed = True
                return True

        elif self.Position() < deadzone:
            self.pressed = False

        return False


#Controls used in engine!
a = Button("key:W", "joy:0:buttons:3")     
b = Button("key:A", "joy:0:buttons:2")     
c = Button("key:S", "joy:0:buttons:1")      
d = Button("key:D", "joy:0:buttons:0")      
e = Button("key:SPACE", "joy:0:buttons:4")
up = Button("key:UP", "joy:0:axis:1-")
down = Button("key:DOWN", "joy:0:axis:1+")
left = Button("key:LEFT", "joy:0:axis:0-")
right = Button("key:RIGHT", "joy:0:axis:0+")

buttons = {"attack": a, "skill_a": b, "skill_b": c, "skill_c" : d,  "guard" : e,"up": up, "down": down, "left": left, "right": right}
