from igl.db import Model
from igl.db.fields import *

class Sprite(Model):
	
	Name = StringField()
	Image = StringField()
	
	Width = IntegerField(min=0, max=500)
	Height = IntegerField(min=0, max=500)
	
	FrameNumber = IntegerField(min=0, max=500)
	
	UpScript = StringField(width=500)
	DownScript = StringField(width=500)
	LeftScript = StringField(width=500)
	RightScript = StringField(width=500)
	
	def __unicode__(self):

		return unicode(self.Name)

class MonsterSprite(Model):
	
	Name = StringField()
	Image = StringField()
	
	Width = IntegerField(min=0, max=500)
	Height = IntegerField(min=0, max=500)
	
	BarX = IntegerField(min=0, max=500)
	BarY = IntegerField(min=0, max=500)
	
	DamageX = IntegerField(min=0, max=500)
	DamageY = IntegerField(min=0, max=500)
	
	FrameNumber = IntegerField(min=0, max=500)
	
	IdleScript = StringField(width=500)
	AttackScript = StringField(width=500)
	OpenScript = StringField(width=500)
	
	def __unicode__(self):

		return unicode(self.Name)
	
class BattleEffect(Model):
	
	Name = StringField()
	Image = StringField()	
	
	Width = IntegerField(min=0, max=500)
	Height = IntegerField(min=0, max=500)
	
	FrameNumber = IntegerField(min=0, max=500)
	
	AnimationScript = StringField(width=500)
	
	def __unicode__(self):

		return unicode(self.Name)

class StatusEffect(Model):
	
	Name = StringField()
	
	Strength = IntegerField(min=-900, max=900)
	Defense = IntegerField(min=-900, max=900)
	Speed = IntegerField(min=-900, max=900)
	
	Power = IntegerField(min=-500, max=500)
	Turns = IntegerField(min=0, max=1000)
	
	PowerFormula = EnumField(['Power', 'Str*2', 'Str/2','Str','Int','Int*2','Int/2','Int to Int*1.5', 'Int/2 + Power'])
	
	EndString = StringField(width=400)
	ProcessString = StringField(width=400)
	
	Effect = EnumField(['None', 'Confuse', 'Poison'])
	
	Image = StringField()
	BattleEffect = ForeignField([BattleEffect], newmodel=True)
	
	def __unicode__(self):

		return unicode(self.Name)
	
class ActiveSkill(Model):
	
	Name = StringField()
	Description = StringField(multiline=True, height=50, width=310)
	
	AttackString = StringField(width=400)
	
	FailString = StringField(width=400)
	SuccessString = StringField(width=400)
	FailString = StringField(width=400)
	
	ChargeString = StringField(width=400)
	
	Target = EnumField(['Enemy', 'AllEnemy', 'Player', 'Random', 'Both'])
	Causes = ForeignField([StatusEffect], newmodel=True)
	Power = IntegerField(min=-500, max=500)
	PowerFormula = EnumField(['Power', 'Str*2', 'Str/2','Str','Int','Int*2','Int/2','Int to Int*1.5', 'Int/2 + Power'])
	Effect = EnumField(['None', 'Damage', 'Cancel', 'Counter', 'Guard', 'Heal', 'Drain','Hydration Damage','Piercing Damage'])
	
	HP_Cost = IntegerField(min=-500, max=500)
	Hydration_Cost = IntegerField(min=-500, max=500)
	
	Cooldown= IntegerField(min=0, max=900)

	BattleEffect = ForeignField([BattleEffect], newmodel=True)
	
	def __unicode__(self):

		return unicode(self.Name+"(%s)" % (self.id) )	

class PassiveSkill(Model):
	
	Name = StringField()
	Description = StringField(multiline=True, height=50, width=310)	
	
	Vitality = IntegerField(min=-100, max=100)
	Strength = IntegerField(min=-100, max=100)
	Defense = IntegerField(min=-100, max=100)
	Speed = IntegerField(min=-100, max=100)
	XP = IntegerField(min=0, max=100)
	
	ConsumptionAdjust = FloatField()
	
	def __unicode__(self):

		return unicode(self.Name)
	
class Item(Model):
	
	Name = StringField()
	Description = StringField(multiline=True, height=50, width=310)	
	
	HPBoost = IntegerField(min=-999,max=999)
	HydrationBoost = IntegerField(min=-999, max=999)
	
	Vitality = IntegerField(min=-100, max=100)
	Strength = IntegerField(min=-100, max=100)
	Defense = IntegerField(min=-100, max=100)
	Speed = IntegerField(min=-100, max=100)
	XP = IntegerField(min=0, max=900)
	
	EffectPower = IntegerField(min=-999,max=999)
	Effect = EnumField(['None', 'SpeedUp', 'Avoid Encounters', 'Key','AllHP'])
	
	Consumable = BooleanField()
	
	def Use(self, player):
		
		c = player.Character
		
		c.Vitality += self.Vitality
		c.HP += self.Vitality
		c.Strength += self.Strength
		c.Defense += self.Defense
		c.Speed += self.Speed
		c.XP += self.XP

		c.HP += self.HPBoost
		if c.HP > c.Vitality:
			c.HP = c.Vitality
			
		c.Hydration += self.HydrationBoost
		
		if self.Effect == 4:
			c.HP = c.Vitality
		elif self.Effect == 2:
			player.AvoidEncounters = self.EffectPower
		elif self.Effect == 1:
			player.SpeedUp = self.EffectPower
	
	def __unicode__(self):

		return unicode(self.Name+"(%s)" % (self.id) )	
	
class Equipment(Model):
	
	Name = StringField()
	Description = StringField(multiline=True, height=50, width=310)		
	
	Type = EnumField(['Sword', 'Armor'])
	Effect = EnumField(['None'])
	
	Vitality = IntegerField(min=-999, max=999)
	Strength = IntegerField(min=-999, max=999)
	Defense = IntegerField(min=-999, max=999)
	Luck = IntegerField(min=-999, max=999)
	Speed = IntegerField(min=-999, max=999)
	
	def __unicode__(self):

		return unicode(self.Name+"(%s)" % (self.id) )	
	
class Monster(Model):	
	
	Name = StringField()
	
	MonsterSprite = ForeignField([MonsterSprite], newmodel=True)
	
	Vitality = IntegerField(min=0, max=900)
	Strength = IntegerField(min=0, max=900)
	Defense = IntegerField(min=0, max=900)
	Speed = IntegerField(min=0, max=900)
	Luck = IntegerField(min=0, max=900)
	XP = IntegerField(min=0, max=900)
	
	Skills = MultiForeignField([ActiveSkill], newmodel=True)
	
	AI = EnumField(['Bird', 'Rock'])
	
	def __unicode__(self):

		return unicode(self.Name+"(%s)" % (self.id) )	
	
	
class EnemyGroup(Model):
	
	Name = StringField()
	
	Enemies = MultiForeignField([Monster], newmodel=True )
	
	x1 = IntegerField()
	y1 = IntegerField()
	
	x2 = IntegerField()
	y2 = IntegerField()
	
	x3 = IntegerField()
	y3 = IntegerField()
	
	x4 = IntegerField()
	y4 = IntegerField()
	
	def __unicode__(self):

		return unicode(self.Name+"(%s)" % (self.id) )	