"""
Ties together the elements of an entity into one object. 
Contains:

Sprite Object - For visual representation of the entity. Contains moving/walking scripts and x/y/layer data.
Character Object - For character representation of the entity. ie, stats, status effects, inventory, etc.
Entity Object (this) - Controls states of the entity, for enemy AI and player input. Also ties together the
                             rest of the entity.

"""

import sprite
import ika
#import character
#from effects import *
import utility 

class Entity:
	
	def __init__(self):
		
		self.Sprite = sprite.Sprite(self)
		#self.Character = character.Character()
		self.State = self.EmptyState().next
		#self.Character = character.Character()
		#self.Sprite.AnimationScript.SetScript("z0 w16 z1 w16 /0")
		
		#self.Sprite.MoveScript.SetScript("u10 l10 r10 d10 w10 /0" )
		
	def Update(self):

		#self.Character.Update()
		self.Sprite.Update()
		
		try:
			return self.State()
		except StopIteration:
			self.State = self.EmptyState()
			return self.State()
		
	def Render(self):
		self.Sprite.Render()
		
	def SetState(self, value):
		"Set a new state. Values passed need to be generator functions"
		
		# If the state can't be overriden, don't change anything.
		#if self.state is not None and not self.interruptableState:
		#	return
		
		if value is None:
			self.State = None
		else:
			try:
				self.State = value.next
			except NameError:
				raise CharacterStateError("Character state is %r, not a generator." % value)
	
	def EmptyState(self):
		"A default state which does nothing and yields nothing."
		while True:
			yield None	

	def DamagedState(self):
		x = 0
		
		while True:
			
			x+=1
			if x == 5:
				self.SetState(self.EmptyState())
				
			yield None

	def OnHurt(self, other):
		#Triggered when the other entity attacks this entity.
		if self.State != self.DamagedState():
			self.SetState(self.DamagedState())

	def OnTouch(self, other):
		#Triggered when the other entity touches this entity.
		pass 