import engine
from controls import *
import windowscript
import textbox

class SkillChoice:
	
	def __init__(self, skill):
		self.Skill = skill
		self.Name = self.Skill.Name
		self.Desc = self.Skill.Description
		
		self.Button = ""
		
		self.ButtonMap = {"skill_a": 2, "skill_b": 3, "skill_c": 4}
		self.ButtonMapN = {"skill_a": "A", "skill_b": "S", "skill_c": "D"}
		
	def Render(self, x, y):
		if self.Button != "":
			engine.Fonts["Order"].Print(x,y,self.ButtonMapN[self.Button])
		engine.Fonts["Interface"].Print(x+10, y, self.Name)
		
	def Update(self, parent):
		pass
	
	def OnSelect(self, parent, button):
		
		if button != "guard" and parent.Dying == False:
		
			if (self.Button == ""):
				if parent.Plugins[0].Player.Character.Skills[self.ButtonMap[button]] is None:
					self.Button = button
					parent.Plugins[0].Player.Character.Skills[self.ButtonMap[button]] = self.Skill
			else:
				parent.Plugins[0].Player.Character.Skills[self.ButtonMap[self.Button]] = None
				self.Button = ""
				
		elif button == "guard" or button == "skill_a" or button == "skill_b" or button == "skill_c":
			
			if parent.Plugins[0].Player.Character.Skills[2] is not None and parent.Plugins[0].Player.Character.Skills[3] is not None and parent.Plugins[0].Player.Character.Skills[4] is not None:
				parent.Close()
				engine.Queue.Add( textbox.TextBox("Press W to open the menu, spacebar to open chests."))
				engine.Queue.Add( textbox.TextBox("In menus A cancels and spacebar confirms."))
				engine.Queue.Add( textbox.TextBox("In battle A, S, D are mapped to your skills. Spacebar defends and W attacks."))
				engine.Queue.Add( textbox.TextBox("As you walk, the gauge below your health will fill. When filled, a battle will occur."))
				engine.Queue.Add( textbox.TextBox("The main route is always north. Caves are side areas that may contain better treasure, but harder enemies."))
			
class StatChoice:
	
	def __init__(self, stat, character, desc):

		self.Amount = getattr(character, stat)
		self.Character = character
		self.Bonus = 0
		
		self.Name = stat
		self.Desc= desc
		self.Multiplier = 1
		
		if (stat == "Luck"):
			self.Multiplier = 2
		
		if (stat == "Vitality"):
			self.Multiplier = 5

	def Render(self, x, y):
		engine.Fonts["Interface"].Print(x, y, self.Name+"("+str(self.Amount)+" + #[FF00FF1D]"+str(self.Bonus)+"#[FFFFFFFF])")
		
	def Update(self, parent):
		pass

	def Increase(self, parent):
		
		if parent.Plugins[0].Points > 0:
		
			self.Bonus += 1
			
			setattr(self.Character,self.Name,self.Amount+(self.Bonus*self.Multiplier)) 
			
			parent.Plugins[0].Points -= 1
		
	def Decrease(self, parent):
		if self.Bonus > 0:
			self.Bonus-=1
			parent.Plugins[0].Points += 1
			setattr(self.Character,self.Name,self.Amount+(self.Bonus*self.Multiplier)) 
	
class ItemChoice:
	
	def __init__(self, index, player):
		self.Index = index
		self.Player = player
		self.Name = self.Player.Items[self.Index]["Item"].Name
		self.Desc = self.Player.Items[self.Index]["Item"].Description
		self.Quantity = self.Player.Items[self.Index]["Quantity"]
		

	def Render(self, x, y):
		
		engine.Fonts["Interface"].Print(x, y, self.Name+" ("+str(self.Quantity)+")")
	
	def Update(self, parent):
		pass
		
	def Finish(self):
		self.Player.CleanInventory()

	def Execute(self, parent):
		if self.Player.Items[self.Index]["Quantity"] > 0:
		
			self.Player.Items[self.Index]["Item"].Use(self.Player)
		
			if self.Player.Items[self.Index]["Item"].Consumable == True:
				self.Player.Items[self.Index]["Quantity"] -= 1
				self.Quantity = self.Player.Items[self.Index]["Quantity"]
			
class EquipChoice:
	
	def __init__(self, item, player):
		self.Item = item
		self.Name = self.Item.Name
		self.Desc = self.Item.Description
		
		self.Equipped = False
		
		if self.Item.Type == 0: #Sword
			if player.Sword.id == self.Item.id:
				self.Equipped = True
		else: #Armor
			if player.Armor.id == self.Item.id:
				self.Equipped = True
		
	def Render(self, x, y):
		if self.Equipped:
			engine.Fonts["Order"].Print(x, y, "E")
			engine.Fonts["Interface"].Print(x+10, y, self.Name)
		else:
			engine.Fonts["Interface"].Print(x, y, self.Name)
			
	def Update(self, parent):
		if self.Item.Type == 0: #Sword
			if parent.Plugins[0].Sword.id == self.Item.id:
				self.Equipped = True
			else:
				self.Equipped = False
		else: #Armor
			if parent.Plugins[0].Armor.id == self.Item.id:
				self.Equipped = True
			else:
				self.Equipped = False

	def Execute(self, parent):
		if self.Item.Type == 0: #Sword
			if parent.Plugins[0].Sword.id != self.Item.id:
				parent.Plugins[0].Sword = self.Item
				self.Equipped = True
		else: #Armor
			if parent.Plugins[0].Armor.id != self.Item.id:
				parent.Plugins[0].Armor = self.Item	
				self.Equipped = True
	
class MenuChoice:
	
	def __init__(self):
		self.Name = "Base choice."
		self.Desc = "Description"
	
	def Render(self, x, y):
		engine.Fonts["Interface"].Print(x, y, self.Name)
		
	def Update(self, parent):
		pass

	def Execute(self, parent):
		pass