from entity import Entity
from controls import *
from direction import *
import sprite
import character
#import projectile
import engine
import math
import battle
import data
import mapgen
import textbox

import zone
import transition
import queue
import skillmenu

class Player(Entity):
	
	LevelOn = 1 #disables leveling while dying/levelling
	AttackReady = 0
	
	def __init__(self,x=None,y=None,character=None):
		self.Sprite = sprite.Sprite(self)
		
		if x == None:
			x = 18*32
		if y == None:
			y = 43*32
		
		self.Encounter = 0
		self.BarOn = 1
		self.HydrationCount = 0
		#self.SkillList = [2,3,4,5,6,7,8,10,11,12,13,14]
		self.SkillList = [8,10,6,11,12,7,3]
		#hp, hyd, vit, str, defen,speed,lck, xp, level
		self.Character = battle.HeroCharacter(30,250,30,4,3,4,8,0,0,[data.ActiveSkill.objects.get(1),data.ActiveSkill.objects.get(9),data.ActiveSkill.objects.get(2),data.ActiveSkill.objects.get(12),data.ActiveSkill.objects.get(4)])
		self.Items = [{"Item": data.Item.objects.get(1), "Quantity": 4}]
		self.Equipment = [data.Equipment.objects.get(1),data.Equipment.objects.get(3)]
		self.Sword = data.Equipment.objects.get(1)
		self.Armor = data.Equipment.objects.get(3)
		
		self.AvoidEncounters = 0
		self.SpeedUp = 0
		
		self.Sprite.x = x
		self.Sprite.y = y
		self.Sprite.Speed = 2
		self.Sprite.Layer = 3
		
		self.Effects = 1
		self.Animations = 1
		self.EncounterList = [0,0,0]
		
		self.State = self.StateStand().next
		
		engine.MapObjects.append(skillmenu.Create(self))
		self.Freeze()
		
	def AddEquip(self, id):

		item = data.Equipment.objects.get(id)
		
		for x in self.Equipment:
			if x.id == id:
				return item.Name #We already have it so don't bother cluttering the player's inventory.
		
		self.Equipment.append(item)
		
		return item.Name
		
	def AddItem(self,id):
		
		item = data.Item.objects.get(id)
		done = 0
		for x in self.Items:
			if x["Item"].id == item.id:
				x["Quantity"]+=1
				done = 1
				
		if not done:
			self.Items.append({"Item": item, "Quantity": 1})
				
		return item.Name

	def GetNextLevel(self):
		#nxp = 30
		#i = 0
		#while i < self.Character.Level+1:
		
		#	nxp = (nxp*.3)+nxp
			
		#	i+=1
		
		Levels = [20,40,80,150,230,250,350,500,600,700,999999999999999]

		return Levels[self.Character.Level]

		#return int(math.floor(nxp))

	def AddExp(self, exp):
		
		self.Character.XP += exp
		self.NextLevels = 0
		while self.Character.XP >= self.GetNextLevel():
			self.Character.XP -= self.GetNextLevel()
			self.Character.Level+=1
			self.NextLevels += 1
		
		#self.NextLevels = 1
		
		return self.NextLevels
		
	def ProcessXP(self):
		engine.MapObjects.append(statmenu.Create(self.NextLevels*3))

	def CleanInventory(self):
		
		for i,x in enumerate(self.Items):
			if x["Quantity"] == 0:
				self.Items.pop(i)


	def Render(self):
		if self.BarOn:
			Entity.Render(self)
		
		#engine.Fonts["Interface"].Print(0,0,str(self.Sprite.x/32))
		#engine.Fonts["Interface"].Print(0,10,str(self.Sprite.y/32))
		#engine.Fonts["Interface"].Print(0,20,str(self.c1.InternalSquare(self.Sprite.x/32,self.Sprite.y/32)))
	
	Color = ika.RGB(0,0,0,255)
	Bar = ika.RGB(205,0,0,255)
	EncBar = ika.RGB(128,0,0)
	BarOn = 1
	Encounter = 0
	EncounterN = 100.0
	w = 80
	h = 8
	h2 = 2

	def RenderBar(self):
		if self.BarOn:
			#HP bar
			engine.Windows["Menu"].Blit(5,5, self.w, self.h,self.Color)
			ika.Video.DrawRect(5-1, 5-1, 5+int((self.w*(self.Character.HP/(self.Character.Vitality*1.0)))), 5+self.h, self.Bar,1)
			engine.Fonts["Interface"].Print(5,5, "%s / %s" % (self.Character.HP, self.Character.Vitality) )
			
			#Encounter bar
			engine.Windows["Menu"].Blit(5,18, self.w, self.h2,self.Color)
			ika.Video.DrawRect(5-1, 18-1, 5+int((self.w*(self.Encounter/(self.EncounterN)))), 18+self.h2, self.EncBar,1)
			
			#Hydration count
			engine.Windows["Menu"].Blit(5,28, 25, self.h,self.Color)
			engine.Fonts["Interface"].Print(5,28, "%s" % (self.Character.Hydration) )	

	def EnableBar(self):
		self.BarOn = 1
		
	def DisableBar(self):
		self.BarOn = 0
		
	def Freeze(self):
		
		self.SetState(self.StateFreeze())
		
	def UnFreeze(self):
		
		self.SetState(self.StateStand())

	def StateStand(self):
		
		while 1:
		
			ika.Map.xwin = int(self.Sprite.x)-(ika.Video.xres/2)
			ika.Map.ywin = int(self.Sprite.y)-(ika.Video.yres/2)			
			
			self.Sprite.MoveScript.SetScriptF( "" )
			self.Sprite.AnimationScript.SetScript( self.Sprite.Animations[self.Sprite.Direction.animation+"_Stand"] )
		
			position, pressed= GetButtons()
			
			if "up" in position or "down" in position or "left" in position or "right" in position:
				self.Sprite.MoveScript.Finished = 1
				self.SetState(self.StateWalk())
				yield None
				
			if "guard" in pressed:
				if self.Sprite.Direction == direction["Up"]:
					if not self.Sprite.CheckTileCollision((self.Sprite.x), (self.Sprite.y)-32,treasure=1):
					
						chest = zone.FindChest((self.Sprite.x/32), (self.Sprite.y/32)-1)
						if chest != None:
							chest.Open(self)
				
				elif self.Sprite.Direction == direction["Down"]:
					if not self.Sprite.CheckTileCollision((self.Sprite.x), (self.Sprite.y)+32,treasure=1):
					
						chest = zone.FindChest((self.Sprite.x/32), (self.Sprite.y/32)+1)
						if chest != None:
							chest.Open(self)					
				elif self.Sprite.Direction == direction["Left"]:
					if not self.Sprite.CheckTileCollision((self.Sprite.x)-32, (self.Sprite.y),treasure=1):
					
						chest = zone.FindChest((self.Sprite.x/32)-1, (self.Sprite.y/32))
						if chest != None:
							chest.Open(self)					
				elif self.Sprite.Direction == direction["Right"]:
					if not self.Sprite.CheckTileCollision((self.Sprite.x)+32, (self.Sprite.y),treasure=1):
					
						chest = zone.FindChest((self.Sprite.x/32)+1, (self.Sprite.y/32))
						if chest != None:
							chest.Open(self)
				
			#if "skill_a" in pressed:
				#battle.StartCombat(0)
			#	self.ResetGame()
			
			#if "skill_b" in pressed:
			#	battle.StartCombat(17)

			#if "skill_c" in pressed:
			#	engine.MapObjects.append(gamemenu.Create())
			
				
			if "attack" in pressed:
				engine.MapObjects.append(gamemenu.Create())
				
			yield None
			
	OldX = -1
	OldY = -1
		
	def StateWalk(self):
		
		while 1:
			
			ika.Map.xwin = self.Sprite.x-(ika.Video.xres/2)
			ika.Map.ywin = self.Sprite.y-(ika.Video.yres/2)			
		
			position, pressed= GetButtons()
		
			if self.Sprite.MoveScript.Finished:
		
				if "up" in position:
					#if "left" in position:
					#	self.Sprite.Direction = direction["UpLeft"]
					#elif "right" in position:
					#	self.Sprite.Direction = direction["UpRight"]
					#else:
						self.Sprite.Direction = direction["Up"]
				elif "down" in position:
					#if "left" in position:
					#	self.Sprite.Direction = direction["DownLeft"]
					#elif "right" in position:
					#	self.Sprite.Direction = direction["DownRight"]
					#else:
						self.Sprite.Direction = direction["Down"]
				elif "left" in position:
					self.Sprite.Direction = direction["Left"]
				elif "right" in position:
					self.Sprite.Direction = direction["Right"]

				else:
					self.SetState(self.StateStand())
					yield None
				
			self.Sprite.Speed = 2
			self.Sprite.AnimationScript.SetScript( self.Sprite.Animations[self.Sprite.Direction.animation+"_Walk"] )
				
			if self.Sprite.MoveScript.SetScriptW( self.Sprite.Direction.movescript+"1" ):
				if  self.OldX != self.Sprite.x or self.OldY != self.Sprite.y:
					self.OldX = self.Sprite.x
					self.OldY = self.Sprite.y
					if zone.CurrentArea[0] == "c":
						self.Encounter += ika.Random(1,9)
						self.HydrationCount += 1
					else:
						self.Encounter += ika.Random(2,7)
						self.HydrationCount += ika.Random(1,3)
					#self.Encounter = 0	
						
					
					if self.HydrationCount >= 10:
						self.Character.Hydration -= 1
						self.HydrationCount = 0
					if self.Encounter > self.EncounterN:
						#ika.Exit(str(self.EncounterList))
						self.Encounter = 0
						self.Sprite.MoveScript.SetScript("")
						self.Sprite.AnimationScript.SetScript( self.Sprite.Animations[self.Sprite.Direction.animation+"_Stand"] )
						self.Freeze()
						battle.StartCombat(self.EncounterList[ika.Random(0,len(self.EncounterList))])
			
			cave = zone.FindCave(self.Sprite.x/32, self.Sprite.y/32)
			map = zone.GetAreaMap(zone.CurrentArea)
			
			if zone.CurrentArea == "1":
				if self.Sprite.y/32 <= 18:
					self.EncounterList = [4,5]
				else:
					self.EncounterList = [2,2,2,2,2,2,3]
					
			elif zone.CurrentArea == "2":
				if self.Sprite.y/32 <= 20:
					self.EncounterList = [12,13,14]
				else:
					self.EncounterList = [9,10,11]
					
			elif zone.CurrentArea[0:2] == "c1":
				self.EncounterList = [7,8]
			elif zone.CurrentArea[0:2] == "c2":
				self.EncounterList = [15,16,17]
			
			if self.Sprite.y == 0:
				if zone.CurrentArea in ["1","2","3","4"]:
					if zone.CurrentArea != "2":
						zone.WarpTo(str(int(zone.CurrentArea)+1))
						self.Freeze()
					else:
						self.Sprite.y += 32
						engine.Queue.Add( textbox.TextBox("Congratulations! You beat the demo. \nLook forward to the REAL game coming soon. \n\nLove, SDHawk & Gizmog1"))
						
			elif self.Sprite.y >= ika.Map.height-32:
				if zone.CurrentArea in ["2","3","4","5"]:
					zone.WarpTo(str(int(zone.CurrentArea)-1),1)
					self.Freeze()
			elif cave != None:
				zone.WarpTo("c"+zone.CurrentArea+"-"+cave.id)
				self.Freeze()

			if zone.CurrentArea[0] == "c":
				if self.Sprite.x/32 == map.x  and self.Sprite.y/32 == map.y+1:
					zone.WarpToXY(zone.CurrentArea[1],zone.OutsideX,zone.OutsideY)
					self.Freeze()					
			
			if self.Character.Hydration <= 0:
				self.Freeze()
				self.ResetGame()

			yield None
			
	def StateFreeze(self):
		
		while 1:
			ika.Map.xwin = int(self.Sprite.x)-(ika.Video.xres/2)
			ika.Map.ywin = int(self.Sprite.y)-(ika.Video.yres/2)	

			#self.Sprite.MoveScript.SetScriptF( "" )
			self.Sprite.AnimationScript.SetScript( self.Sprite.Animations[self.Sprite.Direction.animation+"_Stand"] )
			
			yield None
			
	def ResetGame(self):

		zone.AreaIndex = { }
		zone.TreasureList = {}
		zone.CaveList = {}

		zone.TreasureTotal = 0
		zone.CaveTotal = 0	
		
		engine.Queue.Add( transition.TransitionTo(ika.RGB(255,0,0,0) ) )
		if hasattr(battle,"BM"):
			engine.Queue.Add( queue.Function(battle.StopCombat) )
		engine.Queue.Add( queue.Function(self.__init__))
		engine.Queue.Add( queue.Function(zone.WarpRes,"1") )
		engine.Queue.Add( queue.Function( engine.PlayMusic, "Desert") ,0)
		engine.Queue.Add( transition.TransitionFrom(ika.RGB(255,0,0,0) ) )
		engine.Queue.Add( queue.Function(self.Freeze))

import engine	
import gamemenu
import statmenu
import battle

def GetPlayer():
	
	for map in engine.Maps:
		if map.Name == engine.CurrentMap:	
			for entity in map.Entities:
				if isinstance(entity, Player):
					return entity
					
	return Player()