import ika
import math


def _rotate_point(x, y, angle):
    r = math.hypot(x, y)
    theta = math.atan(float(y) / float(x))
    if x < 0:
        theta += math.pi
    theta += angle
    x = r * math.cos(theta)
    y = r * math.sin(theta)
    return x, y



def RotateBlit(image, cx, cy, angle=0.0, scale=1.0, color=None, blendmode=ika.AlphaBlend, mirror_x=False, mirror_y=False):
    # Check to make sure alpha is > 0.
    if color:
        if ika.GetRGB(color)[3] == 0:
            return

    # To do: use a matrix to make this more efficient.
    angle = angle 
    halfx = image.width / 2
    halfy = image.height / 2

    p=[]
    if mirror_y == False and mirror_x == False:
        p = [_rotate_point(-halfx, -halfy, angle),
             _rotate_point( halfx, -halfy, angle)]
    elif mirror_y == True and mirror_x == False:
        p = [_rotate_point(-halfx, halfy, angle),
             _rotate_point( halfx, halfy, angle)]
    elif mirror_y == False and mirror_x == True:
        p = [_rotate_point(halfx, -halfy, angle),
             _rotate_point(-halfx, -halfy, angle)]
    else:
        p = [_rotate_point(halfx, halfy, angle),
             _rotate_point(-halfx, halfy, angle)]


    p += [[-i for i in p[0]], [-i for i in p[1]]]

    if color == None:
        p = [(int(round(x * scale + cx)), int(round(y * scale + cy))) for x, y in p]
        ika.Video.DistortBlit(image, *p + [blendmode])
    else:
        p = [(int(round(x * scale + cx)), int(round(y * scale + cy)), color) for x, y in p]
        ika.Video.TintDistortBlit(image, *p + [blendmode])



def MirrorBlit(image, x, y, color = ika.RGB(255, 255, 255), mirror_x = False, mirror_y = False):
    coords = (x, y, x + image.width, y + image.height)

    if mirror_x:
        coords = (coords[2], coords[1], coords[0], coords[3])

    if mirror_y:
        pass


    x1, y1, x2, y2 = coords
    ika.Video.TintDistortBlit(image, (x1, y1, color), (x2, y1, color), (x2, y2, color), (x1, y2, color))
