from animationscript import *

class MoveUp:

	String = "u" #The string the command is interpeted as. ie, "u" for up.
	Progress = 0 #How far the window has moved thus far.
	NewX = 0
	NewY = 0
	
	def Execute(self, window, parent, arguments):
		
		self.ApplyMove(window, parent)
		
		window.x = self.NewX
		window.y = self.NewY
			
		self.Progress += window.Speed
			
		if self.Progress >= arguments[0]:
			self.Progress = 0
			parent.Position += 1			

			
	def ApplyMove(self, window, parent):
		
		self.NewX = window.x
		self.NewY = window.y-window.Speed
			
class MoveDown(MoveUp):
	
	String = "d"
	
	def ApplyMove(self, window, parent):
		
		self.NewX = window.x
		self.NewY = window.y+window.Speed
			
class MoveLeft(MoveUp):
	
	String = "l"
	
	def ApplyMove(self, window, parent):
		
		self.NewX = window.x-window.Speed
		self.NewY = window.y
			
			
class MoveRight(MoveUp):
	
	String = "r"
	
	def ApplyMove(self, window, parent):
		
		self.NewX = window.x+window.Speed
		self.NewY = window.y
		
class MoveUpLeft(MoveUp):
	
	String = "ul"
	
	def ApplyMove(self, window, parent):
		
		self.NewX = window.x-window.Speed
		self.NewY = window.y-window.Speed
		
class MoveUpRight(MoveUp):
	
	String = "ur"
	
	def ApplyMove(self, window, parent):
		
		self.NewX = window.x+window.Speed
		self.NewY = window.y-window.Speed
		
class MoveDownLeft(MoveUp):
	
	String = "dl"
	
	def ApplyMove(self, window, parent):
		
		self.NewX = window.x-window.Speed
		self.NewY = window.y+window.Speed
			
class MoveDownRight(MoveUp):
	
	String = "dr"
	
	def ApplyMove(self, window, parent):
		
		self.NewX = window.x+window.Speed
		self.NewY = window.y+window.Speed
		
class MoveSpeed:
	
	String = "s"
	
	def Execute(self, window, parent, arguments):	

		window.Speed = arguments[0]

class SetX:
	
	String = "x"
	
	def Execute(self, window, parent, arguments):	

		window.x = arguments[0]
		
		parent.Position += 1
		
class SetY:
	
	String = "y"
	
	def Execute(self, window, parent, arguments):	

		window.y = arguments[0]
		
		parent.Position += 1
		
class SetXY:
	
	String = "xy"
	
	def Execute(self, window, parent, arguments):	

		window.x = arguments[0]
		window.y = arguments[1]
		
		parent.Position += 1
		
class SetW:
	
	String = "width"
	
	def Execute(self, window, parent, arguments):	

		if len(arguments) == 1 or arguments[1] == 0:

			window.w = arguments[0]
			parent.Position += 1
			
		else:
		
			Total = 0
			
			while Total < arguments[2]:
				if window.w < arguments[0]:
					window.w += 1
				elif window.w > arguments[0]:
					window.w -= 1
				elif window.w == arguments[0]:
					parent.Position += 1
					Total = arguments[2]+1
					
				Total += 1
		
class SetH:
	
	String = "height"
	
	def Execute(self, window, parent, arguments):	
		
		if len(arguments) == 1 or arguments[1] == 0:

			window.h = arguments[0]
			parent.Position += 1
			
		else:
		
			Total = 0
			
			while Total < arguments[2]:
				if window.h < arguments[0]:
					window.h += 1
				elif window.h > arguments[0]:
					window.h -= 1
				elif window.h == arguments[0]:
					parent.Position += 1
					Total = arguments[2]+1
					
				Total += 1
			
		
class SetWH:
	
	String = "wh"
	
	def Execute(self, window, parent, arguments):	

		if len(arguments) == 2 or arguments[2] == 0:

			window.w = arguments[0]
			window.h = arguments[1]
			parent.Position += 1
			
		else:
		
			Total = 0
			
			while Total < arguments[3]:
				if window.w < arguments[0]:
					window.w += 1
				elif window.w > arguments[0]:
					window.w -= 1
					
				if window.h < arguments[1]:
					window.h += 1
				elif window.h > arguments[1]:
					window.h -= 1	
					
				if window.w == arguments[0] and window.h == arguments[1]:
					parent.Position += 1
					Total = arguments[3]+1
					
				Total += 1
		
class SetAlpha:
	
	String = "a"
	
	def Execute(self, window, parent, arguments):
		
		if len(arguments) == 1 or arguments[1] == 0:

			window.Alpha = arguments[0]
			parent.Position += 1
			
		else:
		
			Total = 0
			
			while Total < arguments[2]:
				if window.Alpha < arguments[0]:
					window.Alpha += 1
				elif window.Alpha > arguments[0]:
					window.Alpha -= 1
				elif window.Alpha == arguments[0]:
					parent.Position += 1
					Total = arguments[2]+1
					
				Total += 1

class WindowScript(BaseScript):
	"""
		Window script command list. Use for animating windows. [x] denotes being functional.
		
		
		u<x> - Move up x pixels [x]
		d<x> - Move down x pixels [x]
		l<x> - Move left x pixels [x]
		r<x> - Move right x pixels [x]
		ul<x> - Move up left x pixels [x]
		ur<x> - Move up right x pixels [x]
		dl<x> - Move down left x pixels [x]
		dr<x> - Move down right x pixels [x]
		
		x<x>,<smooth> - Sets window x position to x [x]
		y<x>,<smooth> - Sets window y position to x [x]
		xy<x>,<y>,<smooth> - Sets window x and y positions to x and y. [x]
		
		width<x>,<smooth>,<speed> - Sets window width to x. If smooth is true, it smoothly transitions to the new value at speed. If speed is not specified, movement speed is used.
		height<x>,<smooth>,<speed> - Sets window height to x. If smooth is true, it smoothly transitions to the new value at speed. If speed is not specified, movement speed is used.
		wh<x>,<y>,<smooth>,<speed> - Sets window width and height to x and y. If smooth is true, it smoothly transitions to the new value at speed. If speed is not specified, movement speed is used.
		
		#a<x>,<smooth>,<speed> - Sets window alpha. If smooth is true, it smoothly transitions to the new value at speed. If speed is not specified, movement speed is used.
		(Not actually used since I'm too lazy to code alpha stuff into the windows right now)
		
		s<x> - Set movement speed [x]
		
		w<x> - Wait for x amount of time before proceeding. [x]
		/<x> - Repeat. x = 0 indefintley, x > 0, that many times. [x] 		
	"""
	
	CommandList =  [Wait(), Repeat(),MoveUp(), MoveDown(), MoveRight(), MoveLeft(),
				MoveUpLeft(), MoveUpRight(), MoveDownLeft(), MoveDownRight(),
				MoveSpeed(), SetX(), SetY(), SetXY(), SetW(), SetH(), SetWH(), SetAlpha(), AdjustRGBA() ]
	
	